#pragma once

#include "ProgressWin.h"
#include "RenderMachineSW.h"
#include "DrawModelGLSW.h"
#include "Matrix4.h"
#include "Light.h"
#include "ContourShadingModelSW.h"
#include "ToneShadingModelSW.h"

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Drawing::Imaging;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::Tools;
using namespace IDVR::Tools::OpenGLTools;

namespace IDVR
{
	/**
	<B>GUI class for creating the <i>Animation</i> dialog window and handling the animation calculation.</B>.

	This class is part of the main application namespace IDVR and contains the definition of the <i>Animation</i> dialog window.
	<br><br>
	Furthermore, the entire calculation of the user-defined animation calculation is implemented here. This animation calculation consists
	of the rendering of all needed frames of the resulting video sequence. Note that the video sequence starts at the current position of the 
	image plane and rotate around center of the volume data set by an arbitrary coordinate axis. All rendering setting which includes the choice of the
	render model will be adopted from the main GUI window (see class Neu).
	<br><br>
	This dialog provides several video settting possibilities like:<br><br><ul>
	<li>video length</li><br><br>
	<li>frames per second</li><br><br>
	<li>angle between two consecutive frames</li><br><br>
	<li>angle between two consecutive frames</li><br><br>
	<li>rotation angle of the whole video sequence</li><br><br>
	<li>rotation axis</li></ul>
	<br><br>
	The API documentation for this class shows only the most important member variables and function. Event handling functions and GUI
	components (e.g. labels, textboxes,...) will not be included but all essential parts of the class are surely documented.

	@see Neu - the main GUI window class of this application.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-21-2005
	*/
	public __gc class AnimationWin : public System::Windows::Forms::Form
	{
	public:

		/**
		This is the unique constructor of this class.
		It implements the standard initialization of the GUI window and actually start a 
		thread to create and control a specific dialog window.
		@param n_RayCaster the current instance of the class RenderMachineSW which contains the actual raycaster.
		@param n_VolumeDataDescription the description of the current loaded volume data set.
		@param n_CanvasWidth the width of the current canvas.
		@param n_CanvasHeight the height of the current canvas.
		@param n_Light tje current choosen light model.
		*/
		AnimationWin(RenderMachineSW *n_RayCaster, String *n_VolumeDataDescription, int n_CanvasWidth, int n_CanvasHeight, Light *n_Light)
		{
			InitializeComponent();

			this->Text = String::Concat(MAINTITLE, S" - [loaded dataset] ", n_VolumeDataDescription);
			this->cob_RotationAxis->SelectedIndex = 0;	
			this->teb_SaveDirectory->Text = String::Concat(folderDia_SaveDirectory->SelectedPath, "\\");

			this->m_LengthInSeconds = Convert::ToInt32((this->nud_AnimationScope->Value / this->nud_RotationAngle->Value));
			this->m_FrameCount = this->m_LengthInSeconds * Convert::ToInt32(this->nud_FramesPerSec->Value);
			actualizeStatusBarDisplay();

			this->m_RayCaster = n_RayCaster;
			this->m_CanvasHeight = n_CanvasHeight;
			this->m_CanvasWidth  = n_CanvasWidth;
			this->m_Light = n_Light;
			this->m_SubItem_AnimationEditor = this->m_TbSubItem_AnimationEditor = NULL;
		}
        
	protected: 
		/**
		Destroys all GUI components before this current instance will be actually discarded.
		@param disposing is a flag variable which is true if the instance have to be discarded.
		*/
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}

	private: System::Windows::Forms::GroupBox *  grp_SelectDirectory;
	private: System::Windows::Forms::Button *  but_SaveDirectory;
	private: System::Windows::Forms::TextBox *  teb_SaveDirectory;

	private: System::Windows::Forms::ImageList *  iml_ButtonIcons;
	private: System::Windows::Forms::ComboBox *  cob_RotationAxis;
	private: System::Windows::Forms::GroupBox *  grp_RotationSettings;
	private: System::Windows::Forms::GroupBox *  grp_AnimationSettings;
	private: System::Windows::Forms::NumericUpDown *  nud_AnimationScope;
	private: System::Windows::Forms::Label *  lab_AnimationScope;
	private: System::Windows::Forms::Label *  lab_FrameCount;
	private: System::Windows::Forms::Button *  but_Start;
	private: System::Windows::Forms::Label *  lab_RotationAngle;
	private: System::ComponentModel::IContainer *  components;
	private: System::Windows::Forms::Button *  but_Back;

	private: System::Windows::Forms::NumericUpDown *  nud_RotationAngle;
	private: System::Windows::Forms::RadioButton *  rab_PositiveRotation;
	private: System::Windows::Forms::RadioButton *  rab_NegativeRotation;
	private: System::Windows::Forms::Label *  lab_RotationAxis;
	private: System::Windows::Forms::CheckBox *  chb_LightPosFixed;
	private: System::Windows::Forms::NumericUpDown *  nud_FramesPerSec;
	private: System::Windows::Forms::FolderBrowserDialog *  folderDia_SaveDirectory;
	private: System::Windows::Forms::ToolTip *  tot_ToolsTips;
	private: System::Windows::Forms::StatusBar *  stb_StatusDisplay;
	private: System::Windows::Forms::StatusBarPanel *  stp_FrameCount;
	private: System::Windows::Forms::StatusBarPanel *  stp_LengthInSeconds;
	private: System::Windows::Forms::Label *  lab_AxisOrientation;
	private: System::Windows::Forms::RadioButton *  rab_WorldSystem;
	private: System::Windows::Forms::RadioButton *  rab_ObjectSystem;
	private: System::Windows::Forms::Label *  lab_AxisSystem;
	private: System::Windows::Forms::Panel *  pan_AxisOrientation;
	private: System::Windows::Forms::Panel *  pan_AxisSystem;

	private:

		static String *MAINTITLE = S"Animation Editor", 
					  *FRAMECOUNTTITLE = S"Framecount to be rendered: ", 
					  *LENGHTINSECONDSTITLE = S"Length of Animation (sec): ";

		int m_CanvasWidth;     /**< Saves the current canvas width. */
		int m_CanvasHeight;    /**< Saves the current canvas height. */
		int m_LengthInSeconds; /**< Saves the user-defined video length in seconds. */ 
		int m_FrameCount;      /**< Saves the user-defined overall frame count of the video sequence.*/ 

		TD::SandBar::MenuButtonItem *m_SubItem_AnimationEditor, *m_TbSubItem_AnimationEditor;
		RenderMachineSW *m_RayCaster; /**< Saves the assigned raycaster. 
									       The instance includes all needed rendering settings and the render model. */ 
		Light *m_Light;				  /**< Saves the assigned light model.
									       The instance includes all needed light model settings */ 

		/**
		The main function to create the all needed GUI components.
		Furthermore, the linking of those created GUI components (e.g. labels, textboxes,...) to the current container
		is implemented. After the correct linking all GUI items will be shown on the screen.
		@note This function must not be changed in a code editor because otherwise the GUI builder could be working incorrectly.
		*/
		void InitializeComponent(void)
		{
			this->components = new System::ComponentModel::Container();
			System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(IDVR::AnimationWin));
			this->grp_SelectDirectory = new System::Windows::Forms::GroupBox();
			this->teb_SaveDirectory = new System::Windows::Forms::TextBox();
			this->but_SaveDirectory = new System::Windows::Forms::Button();
			this->iml_ButtonIcons = new System::Windows::Forms::ImageList(this->components);
			this->grp_RotationSettings = new System::Windows::Forms::GroupBox();
			this->pan_AxisSystem = new System::Windows::Forms::Panel();
			this->rab_WorldSystem = new System::Windows::Forms::RadioButton();
			this->rab_ObjectSystem = new System::Windows::Forms::RadioButton();
			this->pan_AxisOrientation = new System::Windows::Forms::Panel();
			this->rab_PositiveRotation = new System::Windows::Forms::RadioButton();
			this->rab_NegativeRotation = new System::Windows::Forms::RadioButton();
			this->lab_AxisSystem = new System::Windows::Forms::Label();
			this->lab_RotationAngle = new System::Windows::Forms::Label();
			this->nud_RotationAngle = new System::Windows::Forms::NumericUpDown();
			this->cob_RotationAxis = new System::Windows::Forms::ComboBox();
			this->lab_AxisOrientation = new System::Windows::Forms::Label();
			this->lab_RotationAxis = new System::Windows::Forms::Label();
			this->grp_AnimationSettings = new System::Windows::Forms::GroupBox();
			this->chb_LightPosFixed = new System::Windows::Forms::CheckBox();
			this->lab_FrameCount = new System::Windows::Forms::Label();
			this->lab_AnimationScope = new System::Windows::Forms::Label();
			this->nud_AnimationScope = new System::Windows::Forms::NumericUpDown();
			this->nud_FramesPerSec = new System::Windows::Forms::NumericUpDown();
			this->but_Start = new System::Windows::Forms::Button();
			this->but_Back = new System::Windows::Forms::Button();
			this->folderDia_SaveDirectory = new System::Windows::Forms::FolderBrowserDialog();
			this->tot_ToolsTips = new System::Windows::Forms::ToolTip(this->components);
			this->stb_StatusDisplay = new System::Windows::Forms::StatusBar();
			this->stp_FrameCount = new System::Windows::Forms::StatusBarPanel();
			this->stp_LengthInSeconds = new System::Windows::Forms::StatusBarPanel();
			this->grp_SelectDirectory->SuspendLayout();
			this->grp_RotationSettings->SuspendLayout();
			this->pan_AxisSystem->SuspendLayout();
			this->pan_AxisOrientation->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_RotationAngle))->BeginInit();
			this->grp_AnimationSettings->SuspendLayout();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_AnimationScope))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_FramesPerSec))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->stp_FrameCount))->BeginInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->stp_LengthInSeconds))->BeginInit();
			this->SuspendLayout();
			// 
			// grp_SelectDirectory
			// 
			this->grp_SelectDirectory->Controls->Add(this->teb_SaveDirectory);
			this->grp_SelectDirectory->Controls->Add(this->but_SaveDirectory);
			this->grp_SelectDirectory->Location = System::Drawing::Point(8, 8);
			this->grp_SelectDirectory->Name = S"grp_SelectDirectory";
			this->grp_SelectDirectory->Size = System::Drawing::Size(432, 56);
			this->grp_SelectDirectory->TabIndex = 0;
			this->grp_SelectDirectory->TabStop = false;
			this->grp_SelectDirectory->Text = S"Directory Selection";
			// 
			// teb_SaveDirectory
			// 
			this->teb_SaveDirectory->BackColor = System::Drawing::SystemColors::HighlightText;
			this->teb_SaveDirectory->Location = System::Drawing::Point(88, 24);
			this->teb_SaveDirectory->Name = S"teb_SaveDirectory";
			this->teb_SaveDirectory->ReadOnly = true;
			this->teb_SaveDirectory->Size = System::Drawing::Size(336, 20);
			this->teb_SaveDirectory->TabIndex = 1;
			this->teb_SaveDirectory->Text = S"";
			// 
			// but_SaveDirectory
			// 
			this->but_SaveDirectory->FlatStyle = System::Windows::Forms::FlatStyle::Popup;
			this->but_SaveDirectory->ImageAlign = System::Drawing::ContentAlignment::BottomLeft;
			this->but_SaveDirectory->ImageIndex = 0;
			this->but_SaveDirectory->ImageList = this->iml_ButtonIcons;
			this->but_SaveDirectory->Location = System::Drawing::Point(8, 24);
			this->but_SaveDirectory->Name = S"but_SaveDirectory";
			this->but_SaveDirectory->Size = System::Drawing::Size(72, 20);
			this->but_SaveDirectory->TabIndex = 0;
			this->but_SaveDirectory->Text = S"Directory";
			this->but_SaveDirectory->TextAlign = System::Drawing::ContentAlignment::MiddleRight;
			this->tot_ToolsTips->SetToolTip(this->but_SaveDirectory, S"Select the folder where all frames should be saved");
			this->but_SaveDirectory->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// iml_ButtonIcons
			// 
			this->iml_ButtonIcons->ImageSize = System::Drawing::Size(16, 16);
			this->iml_ButtonIcons->ImageStream = (__try_cast<System::Windows::Forms::ImageListStreamer *  >(resources->GetObject(S"iml_ButtonIcons.ImageStream")));
			this->iml_ButtonIcons->TransparentColor = System::Drawing::Color::Transparent;
			// 
			// grp_RotationSettings
			// 
			this->grp_RotationSettings->Controls->Add(this->pan_AxisSystem);
			this->grp_RotationSettings->Controls->Add(this->pan_AxisOrientation);
			this->grp_RotationSettings->Controls->Add(this->lab_AxisSystem);
			this->grp_RotationSettings->Controls->Add(this->lab_RotationAngle);
			this->grp_RotationSettings->Controls->Add(this->nud_RotationAngle);
			this->grp_RotationSettings->Controls->Add(this->cob_RotationAxis);
			this->grp_RotationSettings->Controls->Add(this->lab_AxisOrientation);
			this->grp_RotationSettings->Controls->Add(this->lab_RotationAxis);
			this->grp_RotationSettings->Location = System::Drawing::Point(8, 64);
			this->grp_RotationSettings->Name = S"grp_RotationSettings";
			this->grp_RotationSettings->Size = System::Drawing::Size(432, 120);
			this->grp_RotationSettings->TabIndex = 1;
			this->grp_RotationSettings->TabStop = false;
			this->grp_RotationSettings->Text = S"Rotation Settings";
			// 
			// pan_AxisSystem
			// 
			this->pan_AxisSystem->Controls->Add(this->rab_WorldSystem);
			this->pan_AxisSystem->Controls->Add(this->rab_ObjectSystem);
			this->pan_AxisSystem->Location = System::Drawing::Point(248, 44);
			this->pan_AxisSystem->Name = S"pan_AxisSystem";
			this->pan_AxisSystem->Size = System::Drawing::Size(160, 44);
			this->pan_AxisSystem->TabIndex = 10;
			// 
			// rab_WorldSystem
			// 
			this->rab_WorldSystem->Checked = true;
			this->rab_WorldSystem->Location = System::Drawing::Point(8, 4);
			this->rab_WorldSystem->Name = S"rab_WorldSystem";
			this->rab_WorldSystem->Size = System::Drawing::Size(148, 16);
			this->rab_WorldSystem->TabIndex = 8;
			this->rab_WorldSystem->TabStop = true;
			this->rab_WorldSystem->Text = S"align with World System";
			this->tot_ToolsTips->SetToolTip(this->rab_WorldSystem, S"Rotation allign with World Coord System");
			// 
			// rab_ObjectSystem
			// 
			this->rab_ObjectSystem->Location = System::Drawing::Point(8, 24);
			this->rab_ObjectSystem->Name = S"rab_ObjectSystem";
			this->rab_ObjectSystem->Size = System::Drawing::Size(148, 16);
			this->rab_ObjectSystem->TabIndex = 7;
			this->rab_ObjectSystem->Text = S"align with Object System";
			this->tot_ToolsTips->SetToolTip(this->rab_ObjectSystem, S"Rotation align with Object Coord System");
			// 
			// pan_AxisOrientation
			// 
			this->pan_AxisOrientation->Controls->Add(this->rab_PositiveRotation);
			this->pan_AxisOrientation->Controls->Add(this->rab_NegativeRotation);
			this->pan_AxisOrientation->Location = System::Drawing::Point(36, 44);
			this->pan_AxisOrientation->Name = S"pan_AxisOrientation";
			this->pan_AxisOrientation->Size = System::Drawing::Size(128, 44);
			this->pan_AxisOrientation->TabIndex = 9;
			// 
			// rab_PositiveRotation
			// 
			this->rab_PositiveRotation->Checked = true;
			this->rab_PositiveRotation->Location = System::Drawing::Point(8, 4);
			this->rab_PositiveRotation->Name = S"rab_PositiveRotation";
			this->rab_PositiveRotation->Size = System::Drawing::Size(112, 16);
			this->rab_PositiveRotation->TabIndex = 2;
			this->rab_PositiveRotation->TabStop = true;
			this->rab_PositiveRotation->Text = S"positive Rotation";
			// 
			// rab_NegativeRotation
			// 
			this->rab_NegativeRotation->Location = System::Drawing::Point(8, 24);
			this->rab_NegativeRotation->Name = S"rab_NegativeRotation";
			this->rab_NegativeRotation->Size = System::Drawing::Size(112, 16);
			this->rab_NegativeRotation->TabIndex = 1;
			this->rab_NegativeRotation->Text = S"negative Rotation";
			// 
			// lab_AxisSystem
			// 
			this->lab_AxisSystem->Location = System::Drawing::Point(232, 24);
			this->lab_AxisSystem->Name = S"lab_AxisSystem";
			this->lab_AxisSystem->Size = System::Drawing::Size(188, 16);
			this->lab_AxisSystem->TabIndex = 6;
			this->lab_AxisSystem->Text = S"Select the rotation axis system:";
			// 
			// lab_RotationAngle
			// 
			this->lab_RotationAngle->Location = System::Drawing::Point(252, 96);
			this->lab_RotationAngle->Name = S"lab_RotationAngle";
			this->lab_RotationAngle->Size = System::Drawing::Size(108, 16);
			this->lab_RotationAngle->TabIndex = 5;
			this->lab_RotationAngle->Text = S"Rotation Angle/Sec:";
			// 
			// nud_RotationAngle
			// 
			this->nud_RotationAngle->DecimalPlaces = 2;
			System::Int32 __mcTemp__1[] = new System::Int32[4];
			__mcTemp__1[0] = 25;
			__mcTemp__1[1] = 0;
			__mcTemp__1[2] = 0;
			__mcTemp__1[3] = 131072;
			this->nud_RotationAngle->Increment = System::Decimal(__mcTemp__1);
			this->nud_RotationAngle->Location = System::Drawing::Point(368, 92);
			System::Int32 __mcTemp__2[] = new System::Int32[4];
			__mcTemp__2[0] = 360;
			__mcTemp__2[1] = 0;
			__mcTemp__2[2] = 0;
			__mcTemp__2[3] = 0;
			this->nud_RotationAngle->Maximum = System::Decimal(__mcTemp__2);
			System::Int32 __mcTemp__3[] = new System::Int32[4];
			__mcTemp__3[0] = 1;
			__mcTemp__3[1] = 0;
			__mcTemp__3[2] = 0;
			__mcTemp__3[3] = 0;
			this->nud_RotationAngle->Minimum = System::Decimal(__mcTemp__3);
			this->nud_RotationAngle->Name = S"nud_RotationAngle";
			this->nud_RotationAngle->Size = System::Drawing::Size(48, 20);
			this->nud_RotationAngle->TabIndex = 4;
			this->tot_ToolsTips->SetToolTip(this->nud_RotationAngle, S"Select the rotation angle per second (in Grad)");
			System::Int32 __mcTemp__4[] = new System::Int32[4];
			__mcTemp__4[0] = 1;
			__mcTemp__4[1] = 0;
			__mcTemp__4[2] = 0;
			__mcTemp__4[3] = 0;
			this->nud_RotationAngle->Value = System::Decimal(__mcTemp__4);
			this->nud_RotationAngle->ValueChanged += new System::EventHandler(this, perform_ActionNUDValueChanged);
			// 
			// cob_RotationAxis
			// 
			System::Object* __mcTemp__5[] = new System::Object*[3];
			__mcTemp__5[0] = S"X";
			__mcTemp__5[1] = S"Y";
			__mcTemp__5[2] = S"Z";
			this->cob_RotationAxis->Items->AddRange(__mcTemp__5);
			this->cob_RotationAxis->Location = System::Drawing::Point(156, 92);
			this->cob_RotationAxis->Name = S"cob_RotationAxis";
			this->cob_RotationAxis->Size = System::Drawing::Size(48, 21);
			this->cob_RotationAxis->TabIndex = 3;
			this->tot_ToolsTips->SetToolTip(this->cob_RotationAxis, S"Select the rotation axis");
			// 
			// lab_AxisOrientation
			// 
			this->lab_AxisOrientation->Location = System::Drawing::Point(12, 24);
			this->lab_AxisOrientation->Name = S"lab_AxisOrientation";
			this->lab_AxisOrientation->Size = System::Drawing::Size(188, 16);
			this->lab_AxisOrientation->TabIndex = 0;
			this->lab_AxisOrientation->Text = S"Select the rotation axis orientation:";
			// 
			// lab_RotationAxis
			// 
			this->lab_RotationAxis->Location = System::Drawing::Point(12, 96);
			this->lab_RotationAxis->Name = S"lab_RotationAxis";
			this->lab_RotationAxis->Size = System::Drawing::Size(136, 16);
			this->lab_RotationAxis->TabIndex = 2;
			this->lab_RotationAxis->Text = S"Rotation about main axis:";
			// 
			// grp_AnimationSettings
			// 
			this->grp_AnimationSettings->Controls->Add(this->chb_LightPosFixed);
			this->grp_AnimationSettings->Controls->Add(this->lab_FrameCount);
			this->grp_AnimationSettings->Controls->Add(this->lab_AnimationScope);
			this->grp_AnimationSettings->Controls->Add(this->nud_AnimationScope);
			this->grp_AnimationSettings->Controls->Add(this->nud_FramesPerSec);
			this->grp_AnimationSettings->Location = System::Drawing::Point(8, 184);
			this->grp_AnimationSettings->Name = S"grp_AnimationSettings";
			this->grp_AnimationSettings->Size = System::Drawing::Size(432, 48);
			this->grp_AnimationSettings->TabIndex = 2;
			this->grp_AnimationSettings->TabStop = false;
			this->grp_AnimationSettings->Text = S"Animation Settings";
			// 
			// chb_LightPosFixed
			// 
			this->chb_LightPosFixed->Location = System::Drawing::Point(310, 20);
			this->chb_LightPosFixed->Name = S"chb_LightPosFixed";
			this->chb_LightPosFixed->Size = System::Drawing::Size(120, 20);
			this->chb_LightPosFixed->TabIndex = 4;
			this->chb_LightPosFixed->Text = S"Light Position fixed";
			this->tot_ToolsTips->SetToolTip(this->chb_LightPosFixed, S"Select wheter the light should be rotated too");
			// 
			// lab_FrameCount
			// 
			this->lab_FrameCount->Location = System::Drawing::Point(176, 24);
			this->lab_FrameCount->Name = S"lab_FrameCount";
			this->lab_FrameCount->Size = System::Drawing::Size(72, 16);
			this->lab_FrameCount->TabIndex = 3;
			this->lab_FrameCount->Text = S"Frames/Sec:";
			// 
			// lab_AnimationScope
			// 
			this->lab_AnimationScope->Location = System::Drawing::Point(12, 24);
			this->lab_AnimationScope->Name = S"lab_AnimationScope";
			this->lab_AnimationScope->Size = System::Drawing::Size(96, 16);
			this->lab_AnimationScope->TabIndex = 2;
			this->lab_AnimationScope->Text = S"Animation Scope:";
			// 
			// nud_AnimationScope
			// 
			this->nud_AnimationScope->Location = System::Drawing::Point(108, 20);
			System::Int32 __mcTemp__6[] = new System::Int32[4];
			__mcTemp__6[0] = 9999;
			__mcTemp__6[1] = 0;
			__mcTemp__6[2] = 0;
			__mcTemp__6[3] = 0;
			this->nud_AnimationScope->Maximum = System::Decimal(__mcTemp__6);
			System::Int32 __mcTemp__7[] = new System::Int32[4];
			__mcTemp__7[0] = 1;
			__mcTemp__7[1] = 0;
			__mcTemp__7[2] = 0;
			__mcTemp__7[3] = 0;
			this->nud_AnimationScope->Minimum = System::Decimal(__mcTemp__7);
			this->nud_AnimationScope->Name = S"nud_AnimationScope";
			this->nud_AnimationScope->Size = System::Drawing::Size(48, 20);
			this->nud_AnimationScope->TabIndex = 1;
			this->tot_ToolsTips->SetToolTip(this->nud_AnimationScope, S"Select the final rotation range for the animation");
			System::Int32 __mcTemp__8[] = new System::Int32[4];
			__mcTemp__8[0] = 360;
			__mcTemp__8[1] = 0;
			__mcTemp__8[2] = 0;
			__mcTemp__8[3] = 0;
			this->nud_AnimationScope->Value = System::Decimal(__mcTemp__8);
			this->nud_AnimationScope->ValueChanged += new System::EventHandler(this, perform_ActionNUDValueChanged);
			// 
			// nud_FramesPerSec
			// 
			this->nud_FramesPerSec->Location = System::Drawing::Point(248, 20);
			System::Int32 __mcTemp__9[] = new System::Int32[4];
			__mcTemp__9[0] = 35;
			__mcTemp__9[1] = 0;
			__mcTemp__9[2] = 0;
			__mcTemp__9[3] = 0;
			this->nud_FramesPerSec->Maximum = System::Decimal(__mcTemp__9);
			this->nud_FramesPerSec->Name = S"nud_FramesPerSec";
			this->nud_FramesPerSec->Size = System::Drawing::Size(48, 20);
			this->nud_FramesPerSec->TabIndex = 0;
			this->tot_ToolsTips->SetToolTip(this->nud_FramesPerSec, S"Select the count of frames per second");
			System::Int32 __mcTemp__10[] = new System::Int32[4];
			__mcTemp__10[0] = 24;
			__mcTemp__10[1] = 0;
			__mcTemp__10[2] = 0;
			__mcTemp__10[3] = 0;
			this->nud_FramesPerSec->Value = System::Decimal(__mcTemp__10);
			this->nud_FramesPerSec->ValueChanged += new System::EventHandler(this, perform_ActionNUDValueChanged);
			// 
			// but_Start
			// 
			this->but_Start->ImageAlign = System::Drawing::ContentAlignment::BottomLeft;
			this->but_Start->ImageIndex = 3;
			this->but_Start->ImageList = this->iml_ButtonIcons;
			this->but_Start->Location = System::Drawing::Point(128, 240);
			this->but_Start->Name = S"but_Start";
			this->but_Start->Size = System::Drawing::Size(72, 24);
			this->but_Start->TabIndex = 3;
			this->but_Start->Text = S"Start";
			this->tot_ToolsTips->SetToolTip(this->but_Start, S"Start rendering teh animation");
			this->but_Start->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// but_Back
			// 
			this->but_Back->Location = System::Drawing::Point(240, 240);
			this->but_Back->Name = S"but_Back";
			this->but_Back->Size = System::Drawing::Size(72, 24);
			this->but_Back->TabIndex = 4;
			this->but_Back->Text = S"Back";
			this->tot_ToolsTips->SetToolTip(this->but_Back, S"Close the Animation Editor");
			this->but_Back->Click += new System::EventHandler(this, perform_ActionButtonClick);
			// 
			// folderDia_SaveDirectory
			// 
			this->folderDia_SaveDirectory->Description = S"Select the folder for saving all rendered frames";
			this->folderDia_SaveDirectory->RootFolder = Environment::SpecialFolder::MyComputer;
			this->folderDia_SaveDirectory->SelectedPath = S"C:\\burn";
			// 
			// stb_StatusDisplay
			// 
			this->stb_StatusDisplay->Location = System::Drawing::Point(0, 274);
			this->stb_StatusDisplay->Name = S"stb_StatusDisplay";
			System::Windows::Forms::StatusBarPanel* __mcTemp__11[] = new System::Windows::Forms::StatusBarPanel*[2];
			__mcTemp__11[0] = this->stp_FrameCount;
			__mcTemp__11[1] = this->stp_LengthInSeconds;
			this->stb_StatusDisplay->Panels->AddRange(__mcTemp__11);
			this->stb_StatusDisplay->ShowPanels = true;
			this->stb_StatusDisplay->Size = System::Drawing::Size(450, 22);
			this->stb_StatusDisplay->TabIndex = 5;
			// 
			// stp_FrameCount
			// 
			this->stp_FrameCount->Text = S"Framecount to be rendered: ";
			this->stp_FrameCount->ToolTipText = S"The current count of to be rendered frames";
			this->stp_FrameCount->Width = 224;
			// 
			// stp_LengthInSeconds
			// 
			this->stp_LengthInSeconds->Text = S"Length of Animation (sec): ";
			this->stp_LengthInSeconds->ToolTipText = S"The current legth of the animation in seconds";
			this->stp_LengthInSeconds->Width = 224;
			// 
			// AnimationWin
			// 
			this->AutoScaleBaseSize = System::Drawing::Size(5, 13);
			this->ClientSize = System::Drawing::Size(450, 296);
			this->Controls->Add(this->stb_StatusDisplay);
			this->Controls->Add(this->but_Back);
			this->Controls->Add(this->but_Start);
			this->Controls->Add(this->grp_RotationSettings);
			this->Controls->Add(this->grp_SelectDirectory);
			this->Controls->Add(this->grp_AnimationSettings);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->MaximizeBox = false;
			this->Name = S"AnimationWin";
			this->StartPosition = System::Windows::Forms::FormStartPosition::CenterScreen;
			this->Text = S"Animation Editor";
			this->Closing += new System::ComponentModel::CancelEventHandler(this, AnimationWin_Closing);
			this->grp_SelectDirectory->ResumeLayout(false);
			this->grp_RotationSettings->ResumeLayout(false);
			this->pan_AxisSystem->ResumeLayout(false);
			this->pan_AxisOrientation->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_RotationAngle))->EndInit();
			this->grp_AnimationSettings->ResumeLayout(false);
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_AnimationScope))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->nud_FramesPerSec))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->stp_FrameCount))->EndInit();
			(__try_cast<System::ComponentModel::ISupportInitialize *  >(this->stp_LengthInSeconds))->EndInit();
			this->ResumeLayout(false);

		}		

		System::Void perform_ActionButtonClick(System::Object *  sender, System::EventArgs *  e)
		{
			//if the Button SaveDirectory has been clicked
			if (__try_cast<Button*>(sender)->Equals(this->but_SaveDirectory))
			{
				//open the folder dialog
				System::Windows::Forms::DialogResult result = this->folderDia_SaveDirectory->ShowDialog(this);
				if (result == DialogResult::OK) 
				{
					//set the textbox to the new path name
					this->teb_SaveDirectory->Text = String::Concat(folderDia_SaveDirectory->SelectedPath, "\\");
				}

				return;
			}

			//if the Button Start has been clicked
			if (__try_cast<Button*>(sender)->Equals(this->but_Start))
			{
				//Initialize a thread for rendering all needed frames of the animation
				Thread *animationHandler = new Thread(new ThreadStart(this, AnimationWin::renderAnimation));
				animationHandler->set_IsBackground(true);
				animationHandler->Name = S"Animation renderer";
				animationHandler->Start();	
	
				return;
			}

			//if the Button Back has been clicked
			if (__try_cast<Button*>(sender)->Equals(this->but_Back))
			{
				//set the control items of the main IDVR window before close this window
				this->m_SubItem_AnimationEditor->Checked = false;
				this->m_TbSubItem_AnimationEditor->Checked = false;

				this->Dispose(true);
				this->Close();

				return;
			}
		}

		System::Void AnimationWin_Closing(System::Object *  sender, System::ComponentModel::CancelEventArgs *  e)
		{
			this->m_SubItem_AnimationEditor->Checked  = false;
			this->m_TbSubItem_AnimationEditor->Checked = false;
		}

		System::Void perform_ActionNUDValueChanged(System::Object *  sender, System::EventArgs *  e)
		{
			if (__try_cast<NumericUpDown*>(sender)->Equals( this->nud_RotationAngle ))
			{
				//calcualte the new animation parameters
				this->m_LengthInSeconds = Convert::ToInt32((this->nud_AnimationScope->Value / this->nud_RotationAngle->Value));
				this->m_FrameCount = this->m_LengthInSeconds * Convert::ToInt32(this->nud_FramesPerSec->Value);
				actualizeStatusBarDisplay();
				return;
			}
			
			if (__try_cast<NumericUpDown*>(sender)->Equals( this->nud_AnimationScope ))
			{
				//calcualte the new animation parameters
				this->m_LengthInSeconds = Convert::ToInt32((this->nud_AnimationScope->Value / this->nud_RotationAngle->Value));
				this->m_FrameCount = this->m_LengthInSeconds * Convert::ToInt32(this->nud_FramesPerSec->Value);
				actualizeStatusBarDisplay();
				return;
			}

			if (__try_cast<NumericUpDown*>(sender)->Equals( this->nud_FramesPerSec ))
			{
				//calcualte the new animation parameters
				this->m_FrameCount = this->m_LengthInSeconds * Convert::ToInt32(this->nud_FramesPerSec->Value);
				actualizeStatusBarDisplay();
				return;
			}
		}

		/**
		Displays the new calculated user-defined video dettings to the status bar.
		This is just a help for the user to overlook the choosen video settings.
		*/
		void actualizeStatusBarDisplay(void);
		/**
		Actualizes all included render models with the new calculated view and light directions.
		If two-level rendering is selected several local and the global render model must be updated. Otherwise
		only the global render model have to be updated.
		*/
		void actualizeRenderModels(void);
		/**
		Starts the rendering process for all needed frames of the video sequence.
		This function is implemented as a thread to guarantee correct functionality of the other 
		parts of this apllication.
		*/
		void renderAnimation(void);
		
	public:

		//sets the first control item of the main IDVR window
		void setControlSubItem(TD::SandBar::MenuButtonItem *n_SubItem_AnimationEditor);
		//sets the second control item of the main IDVR window
		void setControlTbSubItem(TD::SandBar::MenuButtonItem *n_TbSubItem_AnimationEditor);
};
}
