#include "stdafx.h"
#include "DataLoader.h"

using namespace IDVR;
using namespace IDVR::Volume;
using namespace IDVR::VolumeRenderer;
using namespace IDVR::Interfaces;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ClassificationModels;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::CompositingModels;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels;

/*
This is the standard constructor.
All member variables of this new instance will be set to zero/null.
*/
DataLoader::DataLoader(void)
{
	this->m_Dataset = NULL;
	this->m_GUIHandler1 = NULL;
	this->m_GUIHandler2 = NULL;
	this->m_ProgressbarHandler = NULL;
}

/*
This is the standard deconstructor.
All member variables will be deleted.
*/
DataLoader::~DataLoader(void)
{
	this->m_Dataset = NULL;
	this->m_GUIHandler1 = NULL;
	this->m_GUIHandler2 = NULL;
	this->m_ProgressbarHandler = NULL;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************

/*
Reads the entire volume data structure from file and initialze the actual data structure.
This function encapsulates the main task of this class which is the loading of the volume data structure from
the already specified file name (see DataLoader::set_FileName). The loaded data will be automatically saved in
an instance of the class VolumeData which is the member object DataLoader::m_Dataset.
*/
void DataLoader::readDataFile()
{
	unsigned short volumeDimension __gc[] = new unsigned short __gc[HeaderLength],
				   voxelCoords __gc[] = new unsigned short __gc[3];
	Decimal readingStatusStep = 0.0f, actualReadingStatus = 0.0f;
	bool isProgressbarActive = true;
	FileStream *stream;
	BinaryReader *fileReader;

	try 
	{	
		stream = new FileStream(this->m_FileName, FileMode::Open, FileAccess::Read, FileShare::Read);

		fileReader = new BinaryReader(stream);

		for (int i = 0; i < HeaderLength; i++) 
		{
			volumeDimension[i] = (fileReader->ReadUInt16() & 0x0FFF);
			continue;
		}
		
		readingStatusStep = ((float)(volumeDimension[0] * volumeDimension[1]) / (float)((volumeDimension[0] * volumeDimension[1] * volumeDimension[2]))) * 100.0f;

		clearDataSet();

		this->m_Dataset = new VolumeData(volumeDimension, DataLoader::DensityScope);
		this->m_Dataset->addGeneralObject(DataLoader::GeneralDescription, 
									DataLoader::GeneralImportance, 
									NULL);

		for (short zDim = 0; zDim < volumeDimension[Z]; zDim++)
		//for (short zDim = volumeDimension[2] - 1; zDim >= 0; zDim--)
		{
			//for (short yDim = 0; yDim < volumeDimension[Y]; yDim++)
			for (short yDim = volumeDimension[1] - 1; yDim >= 0; yDim--)
				for (short xDim = 0; xDim < volumeDimension[0]; xDim++) 
				{
					voxelCoords[0] = xDim; voxelCoords[1] = yDim; voxelCoords[2] = zDim;
					try 
					{
						this->m_Dataset->addVoxel(voxelCoords,(fileReader->ReadUInt16() & 0x0FFF));
					}
					catch (Exception* e)
					{ 
						throw new Exception(e->get_Message(), e); 
					}
				}

			//Set the renderstatus to active progressbar
			actualReadingStatus = actualReadingStatus + readingStatusStep;
			if (actualReadingStatus > 1.0f && isProgressbarActive)
			{
				try
				{
					if (!this->m_ProgressbarHandler->isWindowActive())
						throw new Exception();

					int count = Convert::ToInt32(actualReadingStatus / 1.0f);
					for (int i = 0; i < count; i++)
					{
						this->m_ProgressbarHandler->setProgressbarStep();
						this->m_ProgressbarHandler->Refresh();
						actualReadingStatus = actualReadingStatus - 1.0f;
					}
				}
				catch(Exception*) 
				{ 
					fileReader->Close();
					stream->Close();
					this->m_ProgressbarHandler->Close();
					clearDataSet();
					this->m_GUIHandler1 = NULL;
					if (this->m_GUIHandler2 != NULL)
						this->m_GUIHandler2->Start();
					return;
				}
			}
		}
		this->m_ProgressbarHandler->setProgressbarToMaximum();

		fileReader->Close();
		stream->Close();

		this->m_GUIHandler2 = NULL;
		this->m_ProgressbarHandler->Hide();

		short currentMemorySize = (this->m_Dataset->VoxelCountOverall * DataLoader::NumOfBytesPerVoxel) / 1000000;
		if (currentMemorySize <= DataLoader::MaxUsableMemory)
		{
			calculateGradients();
			this->m_Dataset->set_GradientsPreCalculated(true);
		}

		if (this->m_GUIHandler1 != NULL)
			this->m_GUIHandler1->Start();
	} 
	catch ( EndOfStreamException* end) 
	{
		throw new Exception(end->get_Message(), end);
	}
	catch (Exception* e) 
	{	
		throw new Exception(e->get_Message(), e);
	}
	__finally 
	{
		fileReader->Close();
	    stream->Close();
		if (m_ProgressbarHandler)
		{
			this->m_ProgressbarHandler->Close();
			this->m_ProgressbarHandler = NULL;
		}
	}

	//Call the garbage collector to clean up any memory leaks
	GC::Collect();
	GC::WaitForPendingFinalizers();
}

/*
Reads the mask data from file and sets the corresponding voxel object ID numbers.
This function loads the mask data from the already specified file name (see DataLoader::set_FileName) and updates 
the current member object DataLoader::m_Dataset. This means a new volume data object has to be created and the 
corresponding voxel object ID keys have to be set.
*/
unsigned short DataLoader::readMaskFile(String* n_ObjectDescription)
{
	bool isProgressbarActive = true;

	FileStream *stream;
	BinaryReader *fileReader; Byte info, bitMask;
	
	unsigned short voxelCoords __gc[] = new unsigned short __gc[3], 
		           volumeDimension __gc[] = this->m_Dataset->get_VolumeDimension(), 
		           objectID;

	int voxelCountPerSclice = volumeDimension[X] * volumeDimension[Y],
		currentFilePos = 0,
		voxelInSlice = 0,
		readingThreshold = 0;

	try 
	{
		stream = new FileStream(this->m_FileName, FileMode::Open, FileAccess::Read);
		fileReader = new BinaryReader(stream);

		readingThreshold = Convert::ToInt32(Math::Round((float)(fileReader->BaseStream->Length) * 0.01f));
	
		objectID = this->m_Dataset->addObject(n_ObjectDescription, 
										DataLoader::GeneralImportance, 
										NULL);
	
		for (int fileIndex = 0; fileIndex < fileReader->BaseStream->Length; fileIndex++)
		{
			try
			{
				info = fileReader->ReadByte();

				for (int offset = 0; offset < 8; offset++)
				{
					bitMask = 0x80 >> offset;
					if ((info & bitMask) != 0)
					{
						currentFilePos = fileIndex * 8;
						voxelInSlice = (currentFilePos + offset) % voxelCountPerSclice;

						voxelCoords[Z] = (currentFilePos + offset) / voxelCountPerSclice;
						voxelCoords[Y] = voxelInSlice / volumeDimension[X];
						voxelCoords[X] = voxelInSlice % volumeDimension[X];

						this->m_Dataset->VoxelDataset->setVoxelObjectID(voxelCoords, objectID);
						this->m_Dataset->getObjectMember(objectID)->get_DensityHistogram()
								->addNewValue(this->m_Dataset->VoxelDataset->getVoxelDensity(voxelCoords));
					}
				}
			}
			catch ( EndOfStreamException*)
			{ 
				break;
			}
			catch ( IndexOutOfRangeException*) 
			{ 
				break;
			}

			//Set the loading status to active progressbar
			if (fileIndex % readingThreshold == 0)
			{
				try
				{
					if (!this->m_ProgressbarHandler->isWindowActive())
						throw new Exception();

					this->m_ProgressbarHandler->setProgressbarStep();
					this->m_ProgressbarHandler->Refresh();
				}
				catch(Exception*) 
				{ 
					fileReader->Close();
					stream->Close();
					this->m_ProgressbarHandler->Close();
					resetObjectIDs();
				}
			}
		}
	}
	catch (Exception* e) 
	{	
		throw new Exception(e->get_Message(), e);
	}
	
	this->m_ProgressbarHandler->setProgressbarToMaximum();

	fileReader->Close();
	stream->Close();
	this->m_ProgressbarHandler->Close();

	//Call the garbage collector to clean up any memory leaks
	GC::Collect();
	GC::WaitForPendingFinalizers();
	return objectID;
}

//Delets the entire data structure for saving memory.
void DataLoader::clearDataSet(void)
{
	if(this->m_Dataset) 
	{ 
		this->m_Dataset = NULL;

		//Call the garbage collector to clean up any memory leaks
		GC::Collect(); 
		GC::WaitForPendingFinalizers();
	}
}

//Resets the object IDs of all voxels of the volume data structure to the general object.
void DataLoader::resetObjectIDs(void)
{
	this->m_Dataset->resetDataset();
}

//Sets a new file name and file path for upcoming loading processes.
void DataLoader::set_FileName(String *n_FileName)
{
	this->m_FileName = n_FileName;
}

//Returns the current file name and file path.
String* DataLoader::get_FileName(void)
{
	return this->m_FileName;
}

/***************************************************************************************/
//Control classes for the progress window during the loading process.
//This progress window will be updated by an own thread which is started in the main
//GUI class Neu.
void DataLoader::setProgressbarHandler(ProgressWin *n_ProgressbarHandler)
{
	this->m_ProgressbarHandler = n_ProgressbarHandler;
}

void DataLoader::setGUIHandler1(Thread *n_GUIHandler1)
{
	this->m_GUIHandler1 = n_GUIHandler1;
}

void DataLoader::setGUIHandler2(Thread *n_GUIHandler2)
{
	this->m_GUIHandler2 = n_GUIHandler2;
}

/***************************************************************************************/

//Returns the current volume data structure.
VolumeData* DataLoader::getDataset(void)
{
	return this->m_Dataset;
}

//Sets a new gradient calculation mode for upcoming loading processes.
void DataLoader::set_GradientEstimationMode(unsigned char n_GradientEstimationMode)
{
	this->m_GradientEstimationMode = n_GradientEstimationMode;
}

/*
Calculates all gradient vector values of the volume data structure.
This function will be only used if the current data set is not too big because the saving of the gradients consumes lot
of memory. Thus, if the current data set is actually too big (see DataLoader::MaxUsableMemory for the maximum usable 
memory size) the gradients will be calculated during the rendering process.

The member variable DataLoader::m_GradientEstimationMode gives information about which gradient calculation mode should
be used (defined in the %GradientEstimationSW class).
*/
void DataLoader::calculateGradients(void)
{
	bool isProgressbarActive = false;

	//set progressbar parameters
	if (this->m_ProgressbarHandler)
	{
		if (this->m_GradientEstimationMode == GradientEstimationSW::BY_DIFFERENCES)
			this->m_ProgressbarHandler->resetGUIItems(S"Gradient Calculation Process", S"Gradient values are being calculated by Central Differences.");
		else if (this->m_GradientEstimationMode == GradientEstimationSW::BY_4DREGRESSION)
			this->m_ProgressbarHandler->resetGUIItems(S"Gradient Calculation Process", S"Gradient values are being calculated by 4D Regression.");
		this->m_ProgressbarHandler->Show();
		this->m_ProgressbarHandler->Refresh();
		isProgressbarActive = true;
	} 
	unsigned short volumeDimension __gc[] = this->m_Dataset->get_VolumeDimension();

	GradientEstimationSW *gradientCalc = new GradientEstimationSW();
	float newGradientValues __gc[];

	Decimal readingStatusStep = ((float)(volumeDimension[0] * volumeDimension[1]) / (float)((volumeDimension[0] * volumeDimension[1] * volumeDimension[2]))) * 100.0f,
			actualReadingStatus = 0.0f;

	try
	{
		if (this->m_GradientEstimationMode == gradientCalc->BY_DIFFERENCES)
		{
			for (short depth = 0; depth < volumeDimension[Z]; depth++)
			{
				for (short row = 0; row < volumeDimension[Y]; row++)
					for (short column = 0; column < volumeDimension[X]; column++) 
					{
						newGradientValues = gradientCalc->calculate_NormalEstimation(column, row, depth, this->m_Dataset);
						this->m_Dataset->VoxelDataset->setVoxelGradient(column, row, depth, newGradientValues);
					}

				//Set the renderstatus to active progressbar
				actualReadingStatus = actualReadingStatus + readingStatusStep;
				if (actualReadingStatus > 1.0f && isProgressbarActive)
				{
					if (!this->m_ProgressbarHandler->isWindowActive())
						throw new Exception();

					int count = Convert::ToInt32(actualReadingStatus / 1.0f);
					for (int i = 0; i < count; i++)
					{
						this->m_ProgressbarHandler->setProgressbarStep();
						this->m_ProgressbarHandler->Refresh();
						actualReadingStatus = actualReadingStatus - 1.0f;
					}
				}
			}
		}
		else if (this->m_GradientEstimationMode == gradientCalc->BY_4DREGRESSION)
		{	
			for (short depth = 0; depth < volumeDimension[Z]; depth++)
			{
				for (short row = 0; row < volumeDimension[Y]; row++)
					for (short column = 0; column < volumeDimension[X]; column++) 	
					{
						newGradientValues = gradientCalc->calculate_4DLinerRegressionEstimation(column, row, depth, this->m_Dataset);
						this->m_Dataset->VoxelDataset->setVoxelGradient(column, row, depth, newGradientValues);
					}

				//Set the renderstatus to active progressbar
				actualReadingStatus = actualReadingStatus + readingStatusStep;
				if (actualReadingStatus > 1.0f && isProgressbarActive)
				{
					if (!this->m_ProgressbarHandler->isWindowActive())
						throw new Exception();

					int count = Convert::ToInt32(actualReadingStatus / 1.0f);
					for (int i = 0; i < count; i++)
					{
						this->m_ProgressbarHandler->setProgressbarStep();
						this->m_ProgressbarHandler->Refresh();
						actualReadingStatus = actualReadingStatus - 1.0f;
					}
				}
			}
		}
	}
	catch(Exception*) 
	{ 
		this->m_ProgressbarHandler->Close();
		this->m_Dataset->VoxelDataset->resetVoxelGradients();
	}

	this->m_ProgressbarHandler->setProgressbarToMaximum();
	this->m_ProgressbarHandler->Close();
	this->m_ProgressbarHandler = NULL;
}
	