#pragma once

#include "Transferfunction.h"
#include "Histogram.h"
#include "IRenderer.h"

namespace IDVR {
namespace Volume {
namespace DataStructures {

	using namespace System;
	using namespace System::Collections;
	using namespace IDVR::Interfaces;
	
	/**
	<b>Class for representing a specific volume data object with all its attributes.</b>

	This class is part of the DataStructure namespace which is actually a subnamespace of the main namespace Volume. 
	Furthermore the standard class Object is inherted to the %DataObject class for better controlling of a particular 
	quantity of %DataObject instances (particullary the storing of those instances in standard hashtable is considerably easily).
	<br>
	An instance of this class should be only used for saving a specific volume data object which is part of the volume
	data set. Thus, all needed attributes of such a data object can be accumulated which in particular are:
	<br><ul>
	<li>the description of the current volume data object (DataObject::m_ObjectDescription)</li>
	<li>the importance value of the current volume data object (DataObject::m_Importance)</li>
	<li>the specific density histogram of the current volume data object (DataObject::m_DensityHistogram)</li>
	<li>the specific render model of the current volume data object (DataObject::m_RenderModel)</li>
	</ul><br><br>
	Furthermore, all needed function for controlling those important attributes are defined in this class for a
	compete representation of a own volume data object of the whole data set.
	@see VolumeData - the main datastructure class.
	@see Histogram - the basic histogram class.
	@see IRenderer - the abstract render model interface for controlling a specific render model.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-18-2005
	*/
	public __gc class DataObject : public System::Object
	{
		public:	
				/**
				This is the standard constructor.
				All member variables of this new instance will be set to zero/null.
				*/
				DataObject(void);
				/**
				Constructor initialization by all needed specified parameters.
				All member variables will be set to the corresponding given parameters.
				@param n_ObjectDescription the new object description.
				@param n_Importance the new importance value.
				@param n_DensityScope the valid density scope for this instance (e.g. 4096).
				@param n_Renderer the new render model.
				*/
				DataObject(String *n_ObjectDescription, unsigned short n_Importance, 
						   unsigned short n_DensityScope, IRenderer* n_Renderer);

				/**
				This is the standard deconstructor.
				Espacially the histogram and render model will be deleted.
				*/
				~DataObject(void);

				/**	
				Returns the entire saved histogram as an one dimensional int array.
				Use for fast access to Histogram instance values of the current volume object.
				@return An one dimensional int array.
				*/
				int getHistogramValues(void) __gc[];
				/**
				Returns a spcific density count of the object's histogram.
				@param n_Density the specific density value.
				@return The count of the choosen density value.
				*/
				int getHistogramValue(unsigned short n_Density);
			
				/**
				Returns the current importance value.
				@return The current imporance value.
				*/
				__property unsigned short get_Importance(void);
				/**
				Returns the current object description.
				@return The current object description.
				*/
				__property String* get_ObjectDescription(void);
				/**
				Returns the current density histogram.
				@return The current density histogram as an instance of the Histogram class.
				*/
				__property Histogram* get_DensityHistogram(void);
				/**
				Returns the current render model.
				@return The current render model as an instance of the IRenderer class.
				*/
				__property IRenderer* get_RenderModel(void);
				/**
				Sets a new importance value for the current object.
				@param n_Importance the new importance value.
				*/
				__property void set_Importance( unsigned short n_Importance);
				/**
				Sets a new density histogram for the current object.
				@param n_DensityHistogram the new density histogram as an instance of the Histogram class.
				*/
				__property void set_DensityHistogram(Histogram *n_DensityHistogram);
				/**
				Sets a new render model for the current object.
				@param n_RenderModel the new render model encapsulated in a standard IRenderer instance.
				*/
				__property void set_RenderModel(IRenderer* n_RenderModel);

		private: 
				/**	Represents the current saved importance value for the volume object. */
				unsigned short m_Importance;
				/**	Represents the current saved title for the volume object. */
				String *m_ObjectDescription;
				/**	Represents the current saved instance of Histogram for the volume object. */
				DataStructures::Histogram *m_DensityHistogram;
				/**	Represents the current saved instance of IRenderer for the volume object. */
				IRenderer* m_RenderModel;
	};
}
}
}