#pragma once

#include ".\Vector3.h"
#include ".\Vector2.h"
#include ".\Matrix4.h"
#include ".\VolumeData.h"
#include ".\RaySample.h"
#include ".\FootprintElement.h"
#include ".\ImagePlane.h"
#include ".\InterpolationSW.h"
#include ".\ProgressWin.h"

namespace IDVR {
namespace VolumeRenderer {
namespace RenderPrimitives {

using namespace IDVR;
using namespace IDVR::Tools;
using namespace IDVR::Volume::DataStructures;
using namespace System::Collections;
using namespace IDVR::VolumeRenderer::SWVolumeRenderer::RenderCalculations;

/**<B>Class for the footprint datastructue.</B>

Namespace: IDVR::VolumeRenderer::RenderPrimitives

This class provides the footprint datastructure. It contains a list of FootprintElement elements for each
ray and is also responsible for calculating the countersink geometry and to modifiy the first gradient along
one modified ray.

@see FootprintElement - the FootprintElement datastructure
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/

public __gc class Footprint
{

private:

	ArrayList *m_Footprint[,]; /**< List of footprint elements. */
	ImagePlane* m_ImagePlane; /**< The image plane. */
	double m_SamplingDistance; /**< The sampling distance. */
	VolumeData *m_Data; /**< The dataset. */
	InterpolationSW *m_Interpolation; /**< The interpolation model. */
	short m_ObjectIDWithMaxImportance; /**< the object with the objectID with maximal importance. */
	int m_Width; /**< the width of the footprint. */
	int m_Height; /**< the height of the footprint. */
	double m_DistanceToObjectWithMaxImportance __gc[,]; /**< The distance to the objectID with the maximal importance. */
	double m_Slope; /**< the slope of the countersink geometry. */
	double m_GradientScaleFactor; /**< the gradient scalefactor. */
	double m_HighestDepthValue; /**< the highest depth value of the object with the objectID with maximal importance. */

	/**
	Calculates the objectID with the maximal importance.
	*/
	void calculateObjectIDWithMaxImportance(void);
	/**
	Calculates the highest depth value of the object with the objectID with maximal importance.
	@return The depth value.
	*/
	double calculateHighestDepthValue(void);

public:

	/**
	Standard constructor.
	*/
	Footprint(void);
	/**
	Constructor.
	@param n_Width the width of the image plane.
	@param n_Height the height of the image plane.
	@param n_ImagePlane the image plane.
	@param n_SamplingDistance the sampling distance.
	@param n_Data the dataset.
	*/
	Footprint(int n_Width, int n_Height, ImagePlane* n_ImagePlane, double n_SamplingDistance, VolumeData *n_Data);
	/**
	Standard destructor.
	*/
	~Footprint(void);

	/**
	Creates the Footprint for a ray at the specified location.
	@param n_X the x-value of the ray.
	@param n_Y the y-value of the ray.
	@param n_ExitPoint the exit point of the ray.
	@param n_NumOfSamples the number of samples of the ray.
	@param n_InterpolationMode the interpolation mode.
	*/
	void createFootprint(int n_X, int n_Y, Vector3 *n_ExitPoint, int n_NumOfSamples, unsigned char n_InterpolationMode);

	/**
	Returns the Footprint array.
	@return The Footprint array.
	*/
	ArrayList* getFootprint(void) __gc[,];
	/**
	Returns a Footprint list at the specified location.
	@param n_X the x-value of the Footprint list.
	@param n_Y the y-value of the Footprint list.
	@return The Footprint list.
	*/
	ArrayList* getFootprintArrayList(int n_X, int n_Y);
	/**
	Returns the objectID with the maximal importance.
	@return The objectID with the maximal importance.
	*/
	short getObjectIDWithMaxImportance(void);
	/**
	Returns the distance array to the object with the objectID with the maximal importance.
	@return The distance array to the object with the objectID with the maximal importance.
	*/
	double getDistanceToObjectWithMaxImportance(void) __gc[,];
	/**
	Returns the distance to the object with the objectID with the maximal importance at the spacified location.
	@param n_X x-value.
	@param n_Y y-value.
	@return The distance to the object with the objectID with the maximal importance at the spacified location.
	*/
	double getDistanceToObjectWithMaxImportance(int n_X, int n_Y);

	/**
	Sets the countersink slope.
	@param n_Slope the countersinc slope.
	*/
	void setSlope(double n_Slope);

	/**
	Sets the gradient scalefactor.
	@param n_GradientScaleFactor the gradient scalefactor.
	*/
	void setGradientScaleFactor(double n_GradientScaleFactor);

	/**
	Calculates the distance of the ray's new starting point at the specified location.
	@param n_X x_value of the ray.
	@param n_Y y_value of the ray.
	@return The distance of the ray's new starting point at the specified location.
	*/
	double calculateDistanceOfNewStartingPoint(int n_X, int n_Y);

	/**
	Returns the maximal depth of the object with the maximal importance along one Footprint list item at the specified location.
	@param n_X x-value of the Footprint list item.
	@param n_Y y-value of the Footprint list item.
	@return The depth value.
	*/
	double calculateDepthOfObjectWithMaxImpAlongFootprintAL(int n_X, int n_Y);

	/**
	Calculates the distance array.
	*/
	void calculateDistanceArray(void);
	
	/**
	Modifies the gradient.
	@param n_X x-value of the ray
	@param n_Y y-value of the ray
	@param n_ScaleFactor gradient scalefactor
	@param n_RotMatrix rotation matrix of the image plane
	*/
	Gradient* modifyGradient(int n_X, int n_Y, double n_ScaleFactor, Matrix4 *n_RotMatrix);
};
}
}
}