//PhongModel Shader

//an ALPHA_FLOAT16_ATI texture with all density values of the volume dataset
uniform sampler3D density_tex;

//an RGBA_FLOAT16_ATI texture contains the entry point for each ray and the number of Samples along the ray
uniform sampler2D entryPoint_tex;

//an GL_RGBA8 texture contains the current composited color and opacityvalues along the ray
uniform sampler2D globalBuffer_tex;

//an GL_RGBA8 texture contains the current transfer function
uniform sampler1D transferFunction_tex;

//contains the current background color
uniform vec3 backColor;

//contains the offset from the entry point to the current ray sample coordinates
uniform vec3 rayOffset;

//the current view direction
uniform vec3 viewDir;

//the current number of the ray sample
uniform float currentRaySampleNumber;

//contains the size of the density texture
uniform vec3 densitySize;

//contains the current threshold for opacity and gradient magnitude
uniform struct {
  float opacityThreshold;
  float gradientThreshold;
} thresholds;

//contains all needed light source parameters
uniform struct {
	vec3 lightDir;
	vec3 lightColor;
} light;

//contains the needed parameters for phong model shading
uniform struct {
	float ambient;
	float diffuse;
	float specular;
	float shininess;
	vec2 depthCues; //.x -> k1, .y -> k2
} phongParameters;

//contains the current gradient calculation status (.x -> 1..4dRegression, 0..Central Differences;
//													.y -> 1..trilinear,    0..nearest neighbour)													 
uniform ivec2 gradientFlags;

//internal data definitions

//datastructure to saving the neighbourhood's gradients for gradient calculation
struct gradients {
	vec3 gradient[8];
	vec3 calcGradient;
	float gradientLength;
}; 

//datastructure to saving the neighbourhood's voxel coordinates
struct neighbourhoodVoxels {
	vec3 voxelCoords[8];
	vec3 ceilIndices;
	vec3 floorIndices;
};

const float OUTOFVOLUME = 0.0;
const int REGRESSION = 1, CENTRALDIFFERENCES = 0, TRILINEAR = 1, NEARESTNEIGHBOUR = 0;

void do4DRegression(inout gradients gradientValues, in neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords);
void doCentralDifferences(inout gradients gradientValues, in neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords);

void calculateTrilinear(inout gradients gradientValues, in neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords);
void calculateNearestNeighbour(inout gradients gradientValues, in neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords);

vec4 calculateLightning(in gradients gradientValues, in vec4 currentMaterialValues, in vec3 raySampleCoords);
void calculateNeigbourHood(inout neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords);

void main()
{
	//early ray termination
	if(thresholds.opacityThreshold <= texture2D(globalBuffer_tex, gl_FragCoord.xy).a)
	{
		gl_FragColor = vec4(1.0, 1.0, 1.0, 0.0);
		gl_FragDepth = gl_FragCoord.z - 1.0;
		return;
	}

	//calculate the current ray sample coordinates
	vec4 entryPoint = texture2D(entryPoint_tex, gl_FragCoord.xy);

	//if fragment is no part of the volume set
	if(entryPoint.a == OUTOFVOLUME)
	{
		gl_FragColor = vec4(backColor, 1.0);
		gl_FragDepth = gl_FragCoord.z - 1.0;
		return;
	}
		
	vec3 raySampleCoords = vec3(entryPoint) + rayOffset;

	vec4 fragDensity = texture3D(density_tex, raySampleCoords);
	
	vec4 currentMaterialValues = texture1D(transferFunction_tex, fragDensity.x);
	
	//calculating the indices of the neighbouring voxels
	neighbourhoodVoxels neighbourhood;
	calculateNeigbourHood(neighbourhood, raySampleCoords);
	
	//calculate all gradients of the neighbouring voxels
	gradients gradientValues;
	if(gradientFlags.x == REGRESSION)
		do4DRegression(gradientValues, neighbourhood, raySampleCoords);
	else
		doCentralDifferences(gradientValues, neighbourhood, raySampleCoords);
		
	//interpolate the gradient of the current ray sample
	if(gradientFlags.y == TRILINEAR)
		calculateTrilinear(gradientValues, neighbourhood, raySampleCoords);
	else
		calculateNearestNeighbour(gradientValues, neighbourhood, raySampleCoords);
		
	//checks whether the calculated gradient length is long enough 
	if(thresholds.gradientThreshold > gradientValues.gradientLength)
		//calculate the light intensity by phong model
		gl_FragColor = calculateLightning(gradientValues, currentMaterialValues, raySampleCoords);
	else
		gl_FragColor = vec4(1.0, 1.0, 1.0, 0.0);
		
	//check whether the end of the current ray has been reached
	/* if(entryPoint.a <= currentRaySampleNumber)
		gl_FragDepth = gl_FragCoord.z - 1;
	else
		gl_FragDepth = gl_FragCoord.z + 1; */
  gl_FragDepth = (entryPoint.a <= currentRaySampleNumber) ? gl_FragCoord.z - 1.0 : gl_FragCoord.z + 1.0;
		
}

void doCentralDifferences(inout gradients gradientValues, in neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords)
{	
	vec3 tempCoords1, tempCoords2;
	vec3 nullSize = vec3(0.0,0.0,0.0);
	
	for (int index = 0; index < 8; index++)
	{
		tempCoords1 = neighbourhood.voxelCoords[index] + vec3(1.0,1.0,1.0);
		tempCoords2 = neighbourhood.voxelCoords[index] + vec3(-1.0,-1.0,-1.0);
		
		if(all(lessThan(tempCoords1, densitySize)) && all(greaterThanEqual(tempCoords2, nullSize)))
		{
			gradientValues.gradient[index].x =  
              0.5 * (texture3D(density_tex, vec3(tempCoords1.x, neighbourhood.voxelCoords[index].y, neighbourhood.voxelCoords[index].z)).x
					    - texture3D(density_tex, vec3(tempCoords2.x, neighbourhood.voxelCoords[index].y, neighbourhood.voxelCoords[index].z)).x);
					    
      gradientValues.gradient[index].y =  
              0.5 * (texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, tempCoords1.y, neighbourhood.voxelCoords[index].z)).x
              - texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, tempCoords2.y, neighbourhood.voxelCoords[index].z)).x);
		
      gradientValues.gradient[index].z =  
              0.5 * (texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, neighbourhood.voxelCoords[index].y, tempCoords1.z)).x
              - texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, neighbourhood.voxelCoords[index].y, tempCoords2.z)).x); 
		}
		else
			gradientValues.gradient[index] = vec3(0.0,0.0,0.0);	
	}
}

void do4DRegression(inout gradients gradientValues, in neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords)
{
	
	//precalculate the weights for euklidian distances
	vec2 weightedEuclidanDistance = inversesqrt(vec2(2.0, 3.0));	
	
	vec3 tempCoords1, tempCoords2;
	vec3 nullSize = vec3(0.0,0.0,0.0);
	
	for (int index = 0; index < 8; index++)
	{
		tempCoords1 = neighbourhood.voxelCoords[index] + vec3(1.0,1.0,1.0);
		tempCoords2 = neighbourhood.voxelCoords[index] + vec3(-1.0,-1.0,-1.0);
		
		if(all(lessThan(tempCoords1, densitySize)) && all(greaterThanEqual(tempCoords2, nullSize)))
		{
			float voxelDensities[27];
			
			voxelDensities[0]  = texture3D(density_tex, vec3(tempCoords2.x, tempCoords2.y, tempCoords2.z)).x												          * weightedEuclidanDistance.y;
			voxelDensities[1]  = texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, tempCoords2.y, tempCoords2.z)).x							* weightedEuclidanDistance.x;
			voxelDensities[2]  = texture3D(density_tex, vec3(tempCoords1.x, tempCoords2.y, tempCoords2.z)).x												          * weightedEuclidanDistance.y;
			voxelDensities[3]  = texture3D(density_tex, vec3(tempCoords2.x, neighbourhood.voxelCoords[index].y, tempCoords2.z)).x				  		* weightedEuclidanDistance.x;
			voxelDensities[4]  = texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, neighbourhood.voxelCoords[index].y, tempCoords2.z)).x;
			voxelDensities[5]  = texture3D(density_tex, vec3(tempCoords1.x, neighbourhood.voxelCoords[index].y, tempCoords2.z)).x							* weightedEuclidanDistance.x;
			voxelDensities[6]  = texture3D(density_tex, vec3(tempCoords2.x, tempCoords1.y, tempCoords2.z)).x											          	* weightedEuclidanDistance.y;
			voxelDensities[7]  = texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, tempCoords1.y, tempCoords2.z)).x					  	* weightedEuclidanDistance.x;
			voxelDensities[8]  = texture3D(density_tex, vec3(tempCoords1.x, tempCoords1.y, tempCoords2.z)).x	                                * weightedEuclidanDistance.y;
			voxelDensities[9]  = texture3D(density_tex, vec3(tempCoords2.x, tempCoords2.y, neighbourhood.voxelCoords[index].z)).x							* weightedEuclidanDistance.x;
			voxelDensities[10] = texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, tempCoords2.y, neighbourhood.voxelCoords[index].z)).x;
			voxelDensities[11] = texture3D(density_tex, vec3(tempCoords1.x, tempCoords2.y, neighbourhood.voxelCoords[index].z)).x							* weightedEuclidanDistance.x;
			voxelDensities[12] = texture3D(density_tex, vec3(tempCoords2.x, neighbourhood.voxelCoords[index].y, neighbourhood.voxelCoords[index].z)).x;
			voxelDensities[13] = 0.0; 
			voxelDensities[14] = texture3D(density_tex, vec3(tempCoords1.x, neighbourhood.voxelCoords[index].y, neighbourhood.voxelCoords[index].z)).x;
			voxelDensities[15] = texture3D(density_tex, vec3(tempCoords2.x, tempCoords1.y, neighbourhood.voxelCoords[index].z)).x							* weightedEuclidanDistance.x;
			voxelDensities[16] = texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, tempCoords1.y, neighbourhood.voxelCoords[index].z)).x;
			voxelDensities[17] = texture3D(density_tex, vec3(tempCoords1.x, tempCoords1.y, neighbourhood.voxelCoords[index].z)).x							* weightedEuclidanDistance.x;
			voxelDensities[18] = texture3D(density_tex, vec3(tempCoords2.x, tempCoords2.y, tempCoords1.z)).x												          * weightedEuclidanDistance.y;
			voxelDensities[19] = texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, tempCoords2.y, tempCoords1.z)).x							* weightedEuclidanDistance.x;
			voxelDensities[20] = texture3D(density_tex, vec3(tempCoords1.x, tempCoords2.y, tempCoords1.z)).x											          	* weightedEuclidanDistance.y;
			voxelDensities[21] = texture3D(density_tex, vec3(tempCoords2.x, neighbourhood.voxelCoords[index].y, tempCoords1.z)).x					  	* weightedEuclidanDistance.x;
			voxelDensities[22] = texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, neighbourhood.voxelCoords[index].y, tempCoords1.z)).x;
			voxelDensities[23] = texture3D(density_tex, vec3(tempCoords1.x, neighbourhood.voxelCoords[index].y, tempCoords1.z)).x							* weightedEuclidanDistance.x;
			voxelDensities[24] = texture3D(density_tex, vec3(tempCoords2.x, tempCoords1.y, tempCoords1.z)).x												          * weightedEuclidanDistance.y;
			voxelDensities[25] = texture3D(density_tex, vec3(neighbourhood.voxelCoords[index].x, tempCoords1.y, tempCoords1.z)).x							* weightedEuclidanDistance.x;
			voxelDensities[26] = texture3D(density_tex, vec3(tempCoords1.x, tempCoords1.y, tempCoords1.z)).x												          * weightedEuclidanDistance.y;
			
			gradientValues.gradient[index].x = //Slice with depth z -> voxel nummers 9,11,12,14,15,17 - 10, 13 and 16 fell out because of xk = 0
			/*#9*/  -voxelDensities[9]  + /*#11*/ voxelDensities[11] - /*#12*/ voxelDensities[12] +
			/*#14*/  voxelDensities[14] - /*#15*/ voxelDensities[15] + /*#17*/ voxelDensities[17] - 

			//Slice with depth z-1 -> voxel nummers 0,2,3,5,6,8 - 1, 4 and 7 fell out because of xk = 0
			/*#0*/ voxelDensities[0] + /*#2*/ voxelDensities[2] - /*#3*/ voxelDensities[3] +
			/*#5*/ voxelDensities[5] - /*#6*/ voxelDensities[6] + /*#8*/ voxelDensities[8] -
			
			//Slice with depth z+1 -> voxel nummers 18,20,21,23,24,26 - 19, 22 and 25 fell out because of xk = 0
			/*#18*/ voxelDensities[18] + /*#20*/ voxelDensities[20] - /*#21*/ voxelDensities[21] +
			/*#23*/ voxelDensities[23] - /*#24*/ voxelDensities[24] + /*#26*/ voxelDensities[26];
			
			
			gradientValues.gradient[index].y = //Slice with depth z -> voxel nummers 9,10,11,15,16,17 - 12, 13 and 14 fell out because of yk = 0
			/*#9*/  -voxelDensities[9]  - /*#10*/ voxelDensities[10] - /*#11*/ voxelDensities[11] +
			/*#15*/  voxelDensities[15] + /*#16*/ voxelDensities[16] + /*#17*/ voxelDensities[17] -

			//Slice with depth z-1 -> voxel nummers 0,1,2,6,7,8 - 3, 4 and 5 fell out because of yk = 0
			/*#0*/ voxelDensities[0] - /*#1*/ voxelDensities[1] - /*#2*/ voxelDensities[2] +
			/*#6*/ voxelDensities[6] + /*#7*/ voxelDensities[7] + /*#8*/ voxelDensities[8] - 
			
			//Slice with depth z+1 -> voxel nummers 18,19,20,24,25,26 - 21, 22 and 23 fell out because of yk = 0
			/*#18*/ voxelDensities[18] - /*#19*/ voxelDensities[19] - /*#20*/ voxelDensities[20] +
			/*#24*/ voxelDensities[24] + /*#25*/ voxelDensities[25] + /*#26*/ voxelDensities[26]; 		 
			
			
			gradientValues.gradient[index].z = //Slice with depth z-1 -> voxel nummers 0,1,2,3,4,5,6,7,8
			/*#0*/ -voxelDensities[0] - /*#1*/ voxelDensities[1] - /*#2*/ voxelDensities[2] - 
			/*#3*/  voxelDensities[3] - /*#4*/ voxelDensities[4] - /*#5*/ voxelDensities[5] - 
			/*#6*/  voxelDensities[6] - /*#7*/ voxelDensities[7] - /*#8*/ voxelDensities[8] + 
			
			//Slice with depth z+1 -> voxel nummers 18,19,20,21,22,23,24,25,26
			/*#18*/ voxelDensities[18] + /*#19*/ voxelDensities[19] + /*#20*/ voxelDensities[20] + 
			/*#21*/ voxelDensities[21] + /*#22*/ voxelDensities[22] + /*#23*/ voxelDensities[23] + 
			/*#24*/ voxelDensities[24] + /*#25*/ voxelDensities[25] + /*#26*/ voxelDensities[26];
		}
		else
			gradientValues.gradient[index] = vec3(0.0,0.0,0.0);	
	}
}

void calculateTrilinear(inout gradients gradientValues, in neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords)
{
		
		vec3 tempValue1 = neighbourhood.ceilIndices - raySampleCoords;
		vec3 tempValue2 = raySampleCoords - neighbourhood.floorIndices;
		
		float voxelWeights[8];
		voxelWeights[0] = tempValue1.x * tempValue1.y * tempValue1.z;
		voxelWeights[1] = tempValue2.x * tempValue1.y * tempValue1.z;
		voxelWeights[2] = tempValue1.x * tempValue2.y * tempValue1.z;
		voxelWeights[3] = tempValue2.x * tempValue2.y * tempValue1.z;
		voxelWeights[4] = tempValue1.x * tempValue1.y * tempValue2.z;
		voxelWeights[5] = tempValue2.x * tempValue1.y * tempValue2.z;
		voxelWeights[6] = tempValue1.x * tempValue2.y * tempValue2.z;
		voxelWeights[7] = tempValue2.x * tempValue2.y * tempValue2.z;

		//do trilinear interpolation
		gradientValues.calcGradient = 
						 gradientValues.gradient[0] * voxelWeights[0] +		 
						 gradientValues.gradient[1] * voxelWeights[1] +
						 gradientValues.gradient[2] * voxelWeights[2] +
						 gradientValues.gradient[3] * voxelWeights[3] +
						 gradientValues.gradient[4] * voxelWeights[4] +
						 gradientValues.gradient[5] * voxelWeights[5] +
						 gradientValues.gradient[6] * voxelWeights[6] +
						 gradientValues.gradient[7] * voxelWeights[7];

		
	gradientValues.gradientLength = length(gradientValues.calcGradient);
	gradientValues.calcGradient = normalize(gradientValues.calcGradient);
}

void calculateNearestNeighbour(inout gradients gradientValues, in neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords)
{
	float distances[8], currentDistance = 1000.0;
	int selectedIndex;
	
	distances[0] = distance(neighbourhood.voxelCoords[0], raySampleCoords);
	distances[1] = distance(neighbourhood.voxelCoords[1], raySampleCoords);
	distances[2] = distance(neighbourhood.voxelCoords[2], raySampleCoords);
	distances[3] = distance(neighbourhood.voxelCoords[3], raySampleCoords);
	distances[4] = distance(neighbourhood.voxelCoords[4], raySampleCoords);
	distances[5] = distance(neighbourhood.voxelCoords[5], raySampleCoords);
	distances[6] = distance(neighbourhood.voxelCoords[6], raySampleCoords);
	distances[7] = distance(neighbourhood.voxelCoords[7], raySampleCoords);

	//get the index width the smallest distance
	for (int index = 0; index < 8; index++)
	{	
		if (distances[index] < currentDistance) 
		{
			currentDistance = distances[index]; 
			selectedIndex = index;
		}
	}
	
	gradientValues.calcGradient = gradientValues.gradient[selectedIndex];
	gradientValues.gradientLength = length(gradientValues.calcGradient);
	gradientValues.calcGradient = normalize(gradientValues.calcGradient);
}

vec4 calculateLightning(in gradients gradientValues, in vec4 currentMaterialValues, in vec3 raySampleCoords)
{
	float distanceToImagePlane = distance(raySampleCoords, gl_FragCoord.xyz);
	vec3 reflectionDir = reflect(light.lightDir, gradientValues.calcGradient);
	vec2 depthCueing = phongParameters.depthCues + vec2(1.0, distanceToImagePlane);
	float diffuse = max(dot(light.lightDir,  gradientValues.calcGradient),0.0);
	float specular = 0.0;
	
	vec3 intensity =  phongParameters.ambient * currentMaterialValues.xyz * light.lightColor;
	
	if (diffuse > 0.0)
	{
		specular = max(dot(reflectionDir, viewDir), 0.0);
		specular = pow(specular, phongParameters.shininess);
		
		intensity += light.lightColor * (1.0 /(depthCueing.x + depthCueing.y)) * (phongParameters.diffuse * diffuse +  phongParameters.specular * specular);
	}
	
	intensity = min(intensity, 1.0);
	
	return vec4(intensity, currentMaterialValues.a);
}

void calculateNeigbourHood(inout neighbourhoodVoxels neighbourhood, in vec3 raySampleCoords)
{
	//contains the values .x -> x_down, .y -> y_down, .z -> z_down
	neighbourhood.floorIndices = floor(raySampleCoords);
	//contains the values .x -> x_up, .y -> y_up, .z -> z_up
	neighbourhood.ceilIndices  = ceil(raySampleCoords);
	
	// (x_down, y_down, z_down)
	neighbourhood.voxelCoords[0] = neighbourhood.floorIndices;
	// (x_up, y_down, z_down)
	neighbourhood.voxelCoords[1] = vec3(neighbourhood.ceilIndices.x, neighbourhood.floorIndices.y, neighbourhood.floorIndices.z);
	// (x_down, y_up, z_down)
	neighbourhood.voxelCoords[2] = vec3(neighbourhood.floorIndices.x, neighbourhood.ceilIndices.y, neighbourhood.floorIndices.z);
	// (x_up, y_up, z_down)
	neighbourhood.voxelCoords[3] = vec3(neighbourhood.ceilIndices.x, neighbourhood.ceilIndices.y, neighbourhood.floorIndices.z);
	// (x_down, y_down, z_up)
	neighbourhood.voxelCoords[4] = vec3(neighbourhood.floorIndices.x, neighbourhood.floorIndices.y, neighbourhood.ceilIndices.z);
	// (x_up, y_down, z_up)
	neighbourhood.voxelCoords[5] = vec3(neighbourhood.ceilIndices.x, neighbourhood.floorIndices.y, neighbourhood.ceilIndices.z);
	// (x_down, y_up, z_up)
	neighbourhood.voxelCoords[6] = vec3(neighbourhood.floorIndices.x, neighbourhood.ceilIndices.y, neighbourhood.ceilIndices.z);
	// (x_up, y_up, z_up)
	neighbourhood.voxelCoords[7] = neighbourhood.ceilIndices;
}