#pragma once

#include "Matrix4.h"
#include "Light.h"

using namespace CsGL::OpenGL;
using namespace CsGL::Pointers;
using namespace CsGL::Util;
using namespace System;
using namespace IDVR::Tools;
using namespace IDVR::VolumeRenderer::RenderPrimitives;

namespace IDVR {
namespace Tools {
namespace OpenGLTools {

	using namespace System::Drawing;

	/**
	<b>Class for drawing the light icon onto the canvas.</b>

	This class is part of the OpenGLTools namespace which is actually a subnamespace of the main namespace Tools. 
	<br><br>
	The task of this class is not included in the actual rendering process and the succeeding drawing, but to handle
	the correct display of the light icon an the screen which actually shows the user all information about current
	light color and light direction. The used values for color and direction are, of course, depending on the current 
	choosen light model which the user can controll over the main GUI window (see class Neu).
	<br><br>
	As every other class of the subpackage OpenGLTools this class has the base class OpenGLCode of the CSOpenGL library
	to correctly implement the OpenGL code Which will be handled by the OpenGLPanel which will show the specific drawing 
	results on the screen.<br><br>
	Recapitulatory, the task of this class is the correct drawing of the light icin which is standard length arrow to visualize
	the light direction and color.

	@see OpenGLCanvas - the GUI class for the canvas with its draw control
	@see Neu - the main GUI window which also handles the light model.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-20-2005
	*/
public __gc class LightObjectGL : public OpenGLPanel::OpenGLCode
{

private:

	float m_CanvasBackColor __gc[];  /**< Saves the current canvas background color. */
	//Address pointers for openGL displaylists lists
	unsigned int m_ArrowObject;          /**< Saves the index of the used OpenGL texture object for the light arrow. */
	unsigned int m_OrientationInterface; /**< Saves the index of the used OpenGL texture object for the orientation interface
										 which will be drawn around the light arrow. */
	short m_CanvasLength;	    /**< Saves the width of the current canvas. */
	bool m_IsProxyDrawn;	    /**< Saves the status whether the proxy object has been already drawn. */
	Matrix4 *m_OffsetRotation;   /**< Saves the current offset rotation matrix. */
	Matrix4 *m_BaseRotation;	/**< Saves the current base rotation matrix. */
	Matrix4 *m_DefaultRotation; /**< Saves the current default rotation matrix. */
	Light *m_LightModel;		/**< Saves the current light model with all needed light attributes. 
								     This light attributes are direction and color. */

	unsigned char m_LightViewMode;  /**< Saves the current light view mode. 
									     This folowing light modes are possible:<br><br><ul>
										 <li><i>Absolute view mode</i></li><br><br>
										 <li><i>Relative view mode</i></li></ul>*/


	static float ArrowWidth = 60.0f/512.0f, ArrowHeight = 35.0f/512.0f, ShaftWidth = 30.0f/512.0f, ShaftHeight = 15.0f/512.0f;
	static float ArrowMiddleHeight = 17.5f/512.0f, ArrowStepHeight = 10.0f/512.0f, ArrowDepth = 10.0f/512.0f;
	static float Gap = 10.0f / 512.0f, LineLength = 10.0f / 512.0f;

public:

	static const char ABSOLUTELIGHTVIEW = 0, RELATIVLIGHTVIEW = 1;

	/**
	This is the standard constructor.
	All member variables of this new instance will be set to zero/null.
	*/
	LightObjectGL(void);
	/**
	Constructor initialization with the valid light model.
	All member variables will be set to zero besides the canvas length member variable and the light model
	member variable.
	@param n_CanvasLength the width of the current canvas.
	@param n_LightModel the current user-defined light model.
	*/
	LightObjectGL(int n_CanvasLength, Light *n_LightModel);
	/**
	Constructor initialization with the valid light model and a default rotation.
	All member variables will be set to zero besides the canvas length member variables and the light model
	member variable.
	@param n_CanvasLength the width of the current canvas.
	@param n_LightModel the current user-defined light model.
	@param n_DefaultRotation the default rotation matrix.
	*/
	LightObjectGL(int n_CanvasLength, Light *n_LightModel, Matrix4 *n_DefaultRotation);
	
	/**
	This function overwrites the base init funtion of the class OpenGLCode.
	It contains the actual initialization code in OpenGL and also performs the code.
	*/
	virtual void Init(void);
	/**
	This function overwrites the base draw funtion of the class OpenGLCode.
	It contains the actual drawing code in OpenGL and also performs the code.
	*/
	virtual void Draw(void);

	/**
	Sets a new base rotation matrix.
	@param n_BaseRotation the new base rotation matrix.
	*/
	__property void set_BaseRotation(Matrix4 *n_BaseRotation);
	/**
	Sets a new offset rotation matrix.
	@param n_OffsetRotation the new base offset matrix.
	*/
	__property void set_OffsetRotation(Matrix4 *n_OffsetRotation);
	/**
	Sets a new canvas width for the correct drawing of light arrow and orientation interface.
	@param n_CanvasLength the new base rotation matrix.
	*/
	__property void set_NewCanvasSize(int n_CanvasLength);
	/**
	Sets a new canvas background color.
	@param n_CanvasBackColor the new canvas background color as an instance of the standard color class.
	*/
	__property void set_CanvasBackColor(Color n_CanvasBackColor);
	/**
	Sets the flag whether the proxy object is already drawn.
	@param n_IsProxyDrawn the corresponding bool value.
	*/
	__property void set_ProxyDrawnMode(bool n_IsProxyDrawn);
	/**
	Sets a new light view mode.
	@param n_LightViewMode the new light view mode.
	*/
	__property void set_LightViewMode(unsigned char n_LightViewMode);

	/**
	Returns the current base rotation matrix.
	@return The current base rotation matrix as an instance of the Matrix4 class.
	*/
	__property Matrix4* get_BaseRotation(void);
	/**
	Returns the current offset  rotation matrix.
	@return The current offset rotation matrix as an instance of the Matrix4 class.
	*/
	__property Matrix4* get_OffsetRotation(void);

	/**
	Sets a new light user-defined light model.
	@param n_LightModel the new light model which conatins light direction and color.
	*/
	void setLightModel(Light *n_LightModel);
	/**
	Multiplies the specified offset rotation matrix to the current offset rotation matrix of this instance.
	@param n_OffsetRotation the new offset offset matrix.
	*/
	void modifyOffsetRotation(Matrix4 *n_OffsetRotation);
	/**
	Multiplies the specified base rotation matrix to the current base rotation matrix of this instance.
	@param n_BaseRotation the new base offset matrix.
	*/
	void modifyBaseRotation(Matrix4 *n_BaseRotation);
	/**
	Sets the current base rotation matrix to saved default rotation matrix (default light direction).
	*/
	void resetToBaseLightDirection(void);

private:
	/**
	Defines the standard light arrow in OpenGL as an OpenGL display list.
	*/
	void defineArrowObject(void);
	/**
	Defines the standard light orientation interface in OpenGL as an OpenGL display list.
	*/
	void defineOrientationInterface(void);
};
}
}
}
