#include "StdAfx.h"
#include ".\Matrix3.h"
#using <mscorlib.dll>

using namespace IDVR::Tools;

/*
This is the standard constructor.
This standard constructor initialized all matrix elements with zero.
*/
Matrix3::Matrix3(void)
{
	this->m_MatrixElements = new double __gc[ORDER,ORDER];
}

/*
Constructor initialization by a given array of matrix elements.
This constructor initialized all matrix element with the corresponding elements of the given 2 dimensional array.
*/
Matrix3::Matrix3(double n_MatrixElements __gc[,])
{
	this->m_MatrixElements = n_MatrixElements;
}

/*
Constructor initialization by an existing Matrix3 object.
This constructor initialized all matrix elements with the corresponfing values of the given Matrix3 object. Please
notice that the currently initialized object is not directly linked to the given parameter by the same memory address
but it has its own memory space.
*/
Matrix3::Matrix3(Matrix3 *n_Matrix)
{
	this->m_MatrixElements = n_Matrix->getArray();
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

/*
Use this function for multiply matrices with a given Vector3 object.
A right-side matrix multiplication will be used so that the Vector3 object always is the right sided matrix.
*/
Vector3* Matrix3::multiply_WithPoint(Vector3 *v_Point)
{
	double temp __gc[] = new double __gc[ORDER];
	double temp_value = 0.0;

	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
			temp_value += this->m_MatrixElements[row,column] * v_Point->getItem(column);
		temp[row] = temp_value;
		temp_value = 0.0;
	}

	return new Vector3(temp);
}

/**
This function returns the specific matrix element. This element is specified by the given parameter which 
are the row and column count.
*/
double Matrix3::getItem(int row, int column)
{
	return this->m_MatrixElements[row,column];
}

//Get all matrix elements as two dimensional double array.
double Matrix3::getArray(void) __gc[,]
{
	return this->m_MatrixElements;
}

/*
This function sets a specific matrix element by the given value. 
The matrix element which should be overwrited is specified by the given parameter which 
are the row and column count.
*/
void Matrix3::setItem(int row, int column, double value)
{
	this->m_MatrixElements[row,column] = value;
}

/*
Calculates the determinant of the current matrix.
For the calculation of the determinant the SARRUS rule will be used.
*/
double Matrix3::calculateDeterminant(void)
{
	double det =
		getItem(0,0) * getItem(1,1) * getItem(2,2) +
		getItem(1,0) * getItem(2,1) * getItem(0,2) +
		getItem(2,0) * getItem(0,1) * getItem(1,2) -
	    getItem(2,0) * getItem(1,1) * getItem(0,2) -
		getItem(0,0) * getItem(2,1) * getItem(1,2) -
		getItem(1,0) * getItem(0,1) * getItem(2,2);

	return det;
}

//Multiply the current matrix with a skalar value.
void Matrix3::multiply_WithScalar(double n_Scalar)
{
	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			setItem(row,column,getItem(row,column) * n_Scalar);
		}
	}
}

/*
Returns the transpose matrix of the current matrix.
For the transpose matrix a new instance of the Matrix3 class will be created.
*/
Matrix3* Matrix3::transpose(void)
{	
	Matrix3 *transposeMatrix = new Matrix3();

	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			transposeMatrix->setItem(column, row, getItem(row, column));
		}
	}
	return transposeMatrix;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

/*
This static funtion multiplies the given 3x3 matrices.
An ordinary matrix multiplication will be used and the resultiin 3x3 matrix will be saved to a
new created Matrix3 object.
*/
Matrix3* Matrix3::MultiplyMatices(Matrix3* leftMatrix, Matrix3* rightMatrix)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];
	double temp_value = 0.0;

	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			for (int k = 0; k < ORDER; k++) 
				temp_value += leftMatrix->getItem(row,k) * rightMatrix->getItem(k,column);
			temp[row,column] =	temp_value;
			temp_value = 0.0;
		}
	}
	return new Matrix3(temp);
}

//Creates an newly 3x3 identity matrix.
Matrix3* Matrix3::CreateIdentityMatrix(void)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = 1.0; temp [0,1] = 0.0; temp [0,2] = 0.0;
	temp[1,0] = 0.0; temp [1,1] = 1.0; temp [1,2] = 0.0;
	temp[2,0] = 0.0; temp [2,1] = 0.0; temp [2,2] = 1.0;

	return new Matrix3(temp);
}

/**
Creates a reduced 3x3 Cofactor matrix out of a 4x4 matrix.
With the given parameters the 4x4 matrix (also a parameter) will be reduced to a 3x3 matrix by calculating
the Cofactor matrix.
*/
Matrix3* Matrix3::CreateReducedMatrix(Matrix4 *n_BaseMatrix, unsigned short n_Row, unsigned short n_Column)
{
	Matrix3 *reducedMatrix = new Matrix3();
	unsigned short newRow, newColumn;

	for (int row= 0; row < n_BaseMatrix->ORDER; row++)
	{
		for (int column = 0; column < n_BaseMatrix->ORDER; column++)
		{
			if (row != n_Row && column != n_Column)
			{
				newRow = (row > n_Row) ? row - 1 : row;
				newColumn = (column > n_Column) ? column - 1 : column;

				reducedMatrix->setItem(newRow,newColumn, n_BaseMatrix->getItem(row,column));
			}
		}
	}
	return reducedMatrix;
}
