#pragma once

#include "Matrix4.h"

namespace IDVR {
namespace Tools {

	using namespace System;
	/**
	<B>Class for a 3x3 matrix.</B>
	<p>
	This class is part of the Tools namespace and its target is the handling and computing
	of different linear algebra operations (e.g. matrix multiplications). Basically this class
	should only be used for arbitrary linear operation in the 3 dimensional space, all linear transformation
	like translation, rotation, etc. are part of the Matrix4 class and should be used for such operations 
	(becouse of the explicit use homogenous coordinates). <br>
	Please notice that the logical order of the matrix elements is in ordinary (C++ like) alignment. The following
	brief schema visualizes the valid matix alignment: 
	</p><center>
	<table border="0">
	<tr>
	<td><i>m00</i></td>
	<td><i>m01</i></td>
	<td><i>m02</i></td>
	</tr><tr>
	<td><i>m10</i></td>
	<td><i>m11</i></td>
	<td><i>m12</i></td>
	</tr><tr>
	<td><i>m20</i></td>
	<td><i>m21</i></td>
	<td><i>m22</i></td>
	</tr>
	</table>
	</p><br><p>
	</center>
	The implemented linear operations of this class are of course only a fraction of linear algebra, the minimum
	pool of operations which will be used in the IDVR program.
	</p>
	@see Matrix4 - the 4x4 matrix class for linear transformations.
	@author <i>Bernhard Pflugfelder</i>
	@version 1.0
	@date 02-17-2005
	*/
public __gc class Matrix3
{

private:
	
	double m_MatrixElements __gc[,]; /**<The array of the necessary matrix elements. All the needed nine matrix elements are saved in a double array.*/

public:

	static const unsigned char ORDER = 3; /**<A constant that specifies the matrix order of matices of this class type. */

	/**
	This is the standard constructor.
	This standard constructor initialized all matrix elements with zero.
	*/
	Matrix3(void);
	/**
	Constructor initialization by a specified array of matrix elements.
	This constructor initialized all matrix elements with the corresponding elements of the specified 2 dimensional array.
	@param n_MatrixElements the values for the matrix elements.
	*/
	Matrix3(double n_MatrixElements __gc[,]);
	/**
	Constructor initialization by an existing %Matrix3 object.
	This constructor initialized all matrix elements with the corresponfing values of the specified %Matrix3 object. Please
	notice that the currently initialized object is not directly linked to the specified parameter by the same memory address
	but it has its own memory space.
	@param n_Matrix the %Matrix3 object containing the initialization values.
	*/
	Matrix3(Matrix3 *n_Matrix);

	/**
	Use this function for multiply matrices with a specified Vector3 object.
	A right-side matrix multiplication will be used so that the %Vector3 object always is the right sided <i>matrix</i>.
	@param v_Point the specific %Vector3 object to multiply the current matrix.
	@return The result as a newly created %Vector3 object.
	*/
	Vector3* multiply_WithPoint(Vector3 *v_Point);
	/**
	This function returns the specific matrix element. This element is specified by the specified parameter which 
	are the row and column count.
	@param row the row count of the specific element.
	@param column the column count of the specific element.
	@return The matrix element which is defined by teh parameters.
	@exception Exception instance will be thrown if either the row or column count is not valid.
	*/
	double getItem(int row, int column);
	/**
	Get all matrix elements as two dimensional double array.
	@return The two dimensional array containing the corresponding matrix elements.
	*/
	double getArray(void) __gc[,];
	/**
	This function sets a specific matrix element by the specified value. 
	The matrix element which should be overwrited is specified by the specified parameter which 
	are the row and column count.
	@param row the row count of the specific element.
	@param column the column count of the specific element.
	@param value the new value of the specific element.
	@exception Exception instance will be thrown if either the row or column count is not valid.
	*/
	void setItem(int row, int column, double value);
	/**
	Calculates the determinant of the current matrix.
	For the calculation of the determinant the <i>SARRUS</i> rule will be used.
	@return The calculated determinant value.
	*/
	double calculateDeterminant(void);
	/**
	Multiply the current matrix with a skalar value.
	@param n_Scalar the skalar value to multiply the matrix.
	*/
	void multiply_WithScalar(double n_Scalar);
	/**
	Returns the transpose matrix of the current matrix.
	For the transpose matrix a new instance of the %Matrix3 class will be created.
	@return The transposed matrix as a newly created %Matrix3 object.
	*/
	Matrix3* transpose(void);

	/**
	This static funtion multiplies the specified 3x3 matrices.
	An ordinary matrix multiplication will be used and the resultiin 3x3 matrix will be saved to a
	new created %Matrix3 object.
	@param leftMatrix the left sided matrix.
	@param rightMatrix the right sided matrix.
	@return The resulting 3x3 matrix of the multiplication.
	*/
	static Matrix3* MultiplyMatices(Matrix3* leftMatrix, Matrix3* rightMatrix);
	/**
	Creates an newly 3x3 identity matrix.
	@return The 3x3 identity matrix.
	*/
	static Matrix3* CreateIdentityMatrix(void);
	/**
	Creates a reduced 3x3 Cofactor matrix out of a 4x4 matrix.
	With the specified parameters the 4x4 matrix (also a parameter) will be reduced to a 3x3 matrix by calculating
	the Cofactor matrix.
	@param n_BaseMatrix the original base 4x4 matrix..
	@param n_Row the specific row count of the reduction.
	@param n_Column the specific column count of the reduction.
	@return The resulting 3x3 Cofactor matrix.
	*/
	static Matrix3* CreateReducedMatrix(Matrix4 *n_BaseMatrix, unsigned short n_Row, unsigned short n_Column);
};
}
}
