#include "StdAfx.h"
#include ".\Matrix4.h"
#include ".\Matrix3.h"

#using <mscorlib.dll>

using namespace IDVR::Tools;

/*
This is the standard constructor.
This standard constructor initialized all matrix elements with zero.
*/
Matrix4::Matrix4(void)
{
	this->m_MatrixElements = new double __gc[ORDER,ORDER];
}

/*
Constructor initialization by a specified array of matrix elements.
This constructor initialized all matrix element with the corresponding elements of the specified 2 dimensional array.
*/
Matrix4::Matrix4(double n_MatrixElements __gc[,])
{
	this->m_MatrixElements = new double __gc[ORDER,ORDER];

	for (int row= 0; row < ORDER; row++)
		for (int column = 0; column < ORDER; column++)
			this->m_MatrixElements[row,column] = n_MatrixElements[row,column];
}

/*
Constructor initialization by an existing Matrix4 object.
This constructor initialized all matrix elements with the corresponfing values of the specified Matrix4 object. Please
notice that the currently initialized object is not directly linked to the specified parameter by the same memory address
but it has its own memory space.
*/
Matrix4::Matrix4(Matrix4 *n_Matrix)
{
	double n_MatrixElements __gc[,] = n_Matrix->getArray();
	this->m_MatrixElements = new double __gc[ORDER,ORDER];

	for (int row= 0; row < ORDER; row++)
		for (int column = 0; column < ORDER; column++)
			this->m_MatrixElements[row,column] = n_MatrixElements[row,column];
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

/*
Use this function for multiply matrices with a specified Vector4 object.
A right-side matrix multiplication will be used so that the Vector4 object always is the right sided matrix.
*/
Vector4* Matrix4::multiply_WithPoint(Vector4 *v_Point)
{
	double temp __gc[] = new double __gc[ORDER];
	double temp_value = 0.0;

	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
			temp_value += this->m_MatrixElements[row,column] * v_Point->getItem(column);
		temp[row] = temp_value;
		temp_value = 0.0;
	}

	return new Vector4(temp);
}

/*
This function returns the specific matrix element. This element is specified by the specified parameter which 
are the row and column count.
*/
double Matrix4::getItem(int row, int column)
{
	return this->m_MatrixElements[row,column];
}

//Get all matrix elements as two dimensional double array.
double Matrix4::getArray(void) __gc[,]
{
	return this->m_MatrixElements;
}

/*
This function sets a specific matrix element by the specified value. 
The matrix element which should be overwrited is specified by the specified parameter which 
are the row and column count.
*/
void Matrix4::setItem(int row, int column, double value)
{
	this->m_MatrixElements[row,column] = value;
}

//Get an one dimensional array of the matrix elements.
double Matrix4::getOneDimArray(void) __gc[]
{
	double temp __gc[] = new double __gc[ORDER * ORDER];

	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			temp[(row * ORDER) + column] = this->getItem(row,column);
		}
	}

	return temp;
}

/*
This function returns the negative matrix of the current Matrix4 object. 
This means that the current matrix is multiplied with the skalar -1 which is actually a skalar product.
*/
Matrix4* Matrix4::getNegativeMatrix(void)
{
	Matrix4 *tempValue = new Matrix4(this->m_MatrixElements);
	 
	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			this->m_MatrixElements[row,column] *= -1.0;
		}
	}
	return tempValue;
}

/*
Calculates the determinant of the current matrix.
For the calculation of the determinant the ordinary mathematical method will be used. This method
is based on reducing the 4x4 matrix to a 3x3 matrix which determinant value can be easily calculated.
*/
double Matrix4::calculateDeterminant(void)
{
	double det =
		getItem(0,3) * getItem(1,2) * getItem(2,1) * getItem(3,0)-getItem(0,2) * getItem(1,3) * getItem(2,1) * getItem(3,0)-getItem(0,3) * getItem(1,1) * getItem(2,2) * getItem(3,0)+getItem(0,1) * getItem(1,3) * getItem(2,2) * getItem(3,0)+
		getItem(0,2) * getItem(1,1) * getItem(2,3) * getItem(3,0)-getItem(0,1) * getItem(1,2) * getItem(2,3) * getItem(3,0)-getItem(0,3) * getItem(1,2) * getItem(2,0) * getItem(3,1)+getItem(0,2) * getItem(1,3) * getItem(2,0) * getItem(3,1)+
		getItem(0,3) * getItem(1,0) * getItem(2,2) * getItem(3,1)-getItem(0,0) * getItem(1,3) * getItem(2,2) * getItem(3,1)-getItem(0,2) * getItem(1,0) * getItem(2,3) * getItem(3,1)+getItem(0,0) * getItem(1,2) * getItem(2,3) * getItem(3,1)+
	    getItem(0,3) * getItem(1,1) * getItem(2,0) * getItem(3,2)-getItem(0,1) * getItem(1,3) * getItem(2,0) * getItem(3,2)-getItem(0,3) * getItem(1,0) * getItem(2,1) * getItem(3,2)+getItem(0,0) * getItem(1,3) * getItem(2,1) * getItem(3,2)+
		getItem(0,1) * getItem(1,0) * getItem(2,3) * getItem(3,2)-getItem(0,0) * getItem(1,1) * getItem(2,3) * getItem(3,2)-getItem(0,2) * getItem(1,1) * getItem(2,0) * getItem(3,3)+getItem(0,1) * getItem(1,2) * getItem(2,0) * getItem(3,3)+
		getItem(0,2) * getItem(1,0) * getItem(2,1) * getItem(3,3)-getItem(0,0) * getItem(1,2) * getItem(2,1) * getItem(3,3)-getItem(0,1) * getItem(1,0) * getItem(2,2) * getItem(3,3)+getItem(0,0) * getItem(1,1) * getItem(2,2) * getItem(3,3);

	return det;
}

//Multiply the current matrix with a skalar value.
void Matrix4::multiply_WithScalar(double n_Scalar)
{
	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			setItem(row,column,getItem(row,column) * n_Scalar);
		}
	}
}

/*
Returns the transpose matrix of the current matrix.
For the transpose matrix a new instance of the Matrix4 class will be created.
*/
Matrix4* Matrix4::transpose(void)
{	
	Matrix4 *transposeMatrix = new Matrix4();

	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			transposeMatrix->setItem(column, row, getItem(row, column));
		}
	}
	return transposeMatrix;
}

/**********************************************************************************************************************/
/**********************************************************************************************************************/

/*
Creates a newly translation matrix based on the specified translation vector.
With the values of the Vector3 instance a standard translation matrix will be created.
*/
Matrix4* Matrix4::CreateTranslationMatrix(Vector3* n_Translation)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			if ( row == column ) temp[row,column] = 1.0;
			else if (column == 3 && row != 3 ) temp [row,column] = n_Translation->getItem(row);
			else temp[row,column] = 0.0;
		}
	}
	return new Matrix4(temp);
}

/*
Creates a newly rotaion matrix around the x-axis based on the specified angle.
Please notice that the angle parameter have to be only in radians and not in degrees. 
*/
Matrix4* Matrix4::CreateRotXMatrix(double n_angle)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = 1.0; temp [0,1] = 0.0;                temp [0,2] = 0.0;                  temp[0,3] = 0.0;
	temp[1,0] = 0.0; temp [1,1] = Math::Cos(n_angle); temp [1,2] = -Math::Sin(n_angle);  temp[1,3] = 0.0;
	temp[2,0] = 0.0; temp [2,1] = Math::Sin(n_angle); temp [2,2] = Math::Cos(n_angle);   temp[2,3] = 0.0;
	temp[3,0] = 0.0; temp [3,1] = 0.0;                temp [3,2] = 0.0;                  temp[3,3] = 1.0;

	return new Matrix4(temp);
}

/*
Creates a newly rotaion matrix around the y-axis based on the specified angle.
Please notice that the angle parameter have to be only in radians and not in degrees. 
*/
Matrix4* Matrix4::CreateRotYMatrix(double n_angle)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = Math::Cos(n_angle);  temp [0,1] = 0.0; temp [0,2] = Math::Sin(n_angle); temp[0,3] = 0.0;
	temp[1,0] = 0.0;                 temp [1,1] = 1.0; temp [1,2] = 0.0;                temp[1,3] = 0.0;
	temp[2,0] = -Math::Sin(n_angle); temp [2,1] = 0.0; temp [2,2] = Math::Cos(n_angle); temp[2,3] = 0.0;
	temp[3,0] = 0.0;                 temp [3,1] = 0.0; temp [3,2] = 0.0;                temp[3,3] = 1.0;

	return new Matrix4(temp);
}

/*
Creates a newly rotaion matrix around the z-axis based on the specified angle.
Please notice that the angle parameter have to be only in radians and not in degrees. 
*/
Matrix4* Matrix4::CreateRotZMatrix(double n_angle)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = Math::Cos(n_angle); temp [0,1] = -Math::Sin(n_angle); temp [0,2] = 0.0; temp[0,3] = 0.0;
	temp[1,0] = Math::Sin(n_angle); temp [1,1] = Math::Cos(n_angle);  temp [1,2] = 0.0; temp[1,3] = 0.0;
	temp[2,0] = 0.0;                temp [2,1] = 0.0;                 temp [2,2] = 1.0; temp[2,3] = 0.0;
	temp[3,0] = 0.0;                temp [3,1] = 0.0;                 temp [3,2] = 0.0; temp[3,3] = 1.0;

	return new Matrix4(temp);
}

/*
Calculates the inverted matrix of the specified 4x4 matrix.
The calculation of the inverse matrix will be realized by multiply the Cofactor matrix with the
inverse determinant (just an abrasive try to explain this approximation of the inverse matrix).
*/
Matrix4* Matrix4::InvertMatrix(Matrix4 *n_Matrix)
{
	Matrix4 *inverseMatrix = new Matrix4();

	//Calculate the cofactor matrix from n_Matrix
	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			short coefficient = ((row + column) % 2) ? -1 : 1;
			double coFactorElement = coefficient * (Matrix3::CreateReducedMatrix(n_Matrix->transpose(), row, column))->calculateDeterminant();
			inverseMatrix->setItem(row, column, coFactorElement);
		}
	}

	//Multiply cofactor matrix with the inverse determinant
	inverseMatrix->multiply_WithScalar(1.0 / n_Matrix->calculateDeterminant());

	return inverseMatrix;
}

/*
Creates an uniform scale matrix by the specified scale parameter around the origin.
That means that all dimensions will be scaled by the same factor.
*/
Matrix4* Matrix4::CreateScaleUniformMatrix(double n_ScaleFactor)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = n_ScaleFactor; temp [0,1] = 0.0; temp [0,2] = 0.0; temp[0,3] = 0.0;
	temp[1,0] = 0.0; temp [1,1] = n_ScaleFactor; temp [1,2] = 0.0; temp[1,3] = 0.0;
	temp[2,0] = 0.0; temp [2,1] = 0.0; temp [2,2] = n_ScaleFactor; temp[2,3] = 0.0;
	temp[3,0] = 0.0; temp [3,1] = 0.0; temp [3,2] = 0.0; temp[3,3] = 1.0;

	return new Matrix4(temp);
}

/*
Creates an uniform scale matrix by the specified scale parameter around an arbitrary point.
That means that all dimensions will be scaled by the same factor around a fix point.
*/
Matrix4* Matrix4::CreateScaleUniformMatrix(double n_ScaleFactor, Vector3 *n_FixedPoint)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = n_ScaleFactor; temp [0,1] = 0.0; temp [0,2] = 0.0; temp[0,3] = (1.0 - n_ScaleFactor) * n_FixedPoint->X;
	temp[1,0] = 0.0; temp [1,1] = n_ScaleFactor; temp [1,2] = 0.0; temp[1,3] = (1.0 - n_ScaleFactor) * n_FixedPoint->Y;
	temp[2,0] = 0.0; temp [2,1] = 0.0; temp [2,2] = n_ScaleFactor; temp[2,3] = (1.0 - n_ScaleFactor) * n_FixedPoint->Z;
	temp[3,0] = 0.0; temp [3,1] = 0.0; temp [3,2] = 0.0; temp[3,3] = 1.0;

	return new Matrix4(temp);
}

//Creates a reflection matrix about the x-axis.
Matrix4* Matrix4::CreateReflectionAboutXMatrix(void)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = 1.0;  temp [0,1] =  0.0; temp [0,2] = 0.0; temp[0,3] = 0.0;
	temp[1,0] = 0.0;  temp [1,1] = -1.0; temp [1,2] = 0.0; temp[1,3] = 0.0;
	temp[2,0] = 0.0;  temp [2,1] =  0.0; temp [2,2] = 1.0; temp[2,3] = 0.0;
	temp[3,0] = 0.0;  temp [3,1] =  0.0; temp [3,2] = 0.0; temp[3,3] = 1.0;

	return new Matrix4(temp);
}

//Creates a reflection matrix about the y-axis.
Matrix4* Matrix4::CreateReflectionAboutYMatrix(void)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = -1.0; temp [0,1] = 0.0;  temp [0,2] = 0.0; temp[0,3] = 0.0;
	temp[1,0] =  0.0; temp [1,1] = 1.0; temp [1,2] = 0.0; temp[1,3] = 0.0;
	temp[2,0] =  0.0; temp [2,1] = 0.0;  temp [2,2] = 1.0; temp[2,3] = 0.0;
	temp[3,0] =  0.0; temp [3,1] = 0.0;  temp [3,2] = 0.0; temp[3,3] = 1.0;

	return new Matrix4(temp);
}

//Creates a reflection matrix about the z-axis.
Matrix4* Matrix4::CreateReflectionAboutZMatrix(void)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = 1.0; temp [0,1] = 0.0; temp [0,2] = 0.0;  temp[0,3] = 0.0;
	temp[1,0] = 0.0; temp [1,1] = 1.0; temp [1,2] = 0.0;  temp[1,3] = 0.0;
	temp[2,0] = 0.0; temp [2,1] = 0.0; temp [2,2] = -1.0; temp[2,3] = 0.0;
	temp[3,0] = 0.0; temp [3,1] = 0.0; temp [3,2] = 0.0;  temp[3,3] = 1.0;

	return new Matrix4(temp);
}

/*
This static funtion multiplies the specified 4x4 matrices.
An ordinary matrix multiplication will be used and the resultiin 3x3 matrix will be saved to a
new created %Matrix4 object.
*/
Matrix4* Matrix4::MultiplyMatices(Matrix4* leftMatrix, Matrix4* rightMatrix)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];
	double temp_value = 0.0;

	for (int row= 0; row < ORDER; row++)
	{
		for (int column = 0; column < ORDER; column++)
		{
			for (int k = 0; k < 4; k++) 
				temp_value += leftMatrix->getItem(row,k) * rightMatrix->getItem(k,column);
			temp[row,column] =	temp_value;
			temp_value = 0.0;
		}
	}
	return new Matrix4(temp);
}

//Creates an newly 4x4 identity matrix.
Matrix4* Matrix4::CreateIdentityMatrix(void)
{
	double temp __gc[,] = new double __gc[ORDER,ORDER];

	temp[0,0] = 1.0; temp [0,1] = 0.0; temp [0,2] = 0.0; temp[0,3] = 0.0;
	temp[1,0] = 0.0; temp [1,1] = 1.0; temp [1,2] = 0.0; temp[1,3] = 0.0;
	temp[2,0] = 0.0; temp [2,1] = 0.0; temp [2,2] = 1.0; temp[2,3] = 0.0;
	temp[3,0] = 0.0; temp [3,1] = 0.0; temp [3,2] = 0.0; temp[3,3] = 1.0;

	return new Matrix4(temp);
}