#pragma once

#include "IShadingmodel.h"
#include "Transferfunction.h"
#include "Vector3.h"
#include "Light.h"

namespace IDVR {
namespace VolumeRenderer {
namespace SWVolumeRenderer {
namespace ShadingModels {

using namespace IDVR::Interfaces;
using namespace IDVR::VolumeRenderer::RenderPrimitives;
using namespace IDVR::Tools;
using namespace IDVR::Volume::DataStructures;

/**<B>Class for the phong shading model.</B>

Namespace: IDVR::VolumeRenderer::SWVolumeRenderer::ShadingModels

Baseclass: IDVR::Interfaces::IShadingModel

This class calculates the color value of a given raysample. The color depends on various attributes like ambient part
or specular exponent, but also on values like light color and gradient.

@see ContourShadingModelSW - the contour shading model
@see MipShadingModelSW - the mip shading model
@see ToneShadingModelSW - the tone shading model
@author <i>Christopher Thurnher</i>
@version 1.0
@date 02-18-2005
*/

public __gc class PhongShadingModelSW : public IShadingModel
{
	
private:

	__value enum colorIndices{RED, GREEN, BLUE};

	double k_a; /**< k_a-value. */
	double k_d; /**< k_d-value. */
	double k_s; /**< k_s-value. */
	double n; /**< n-value. */
	double d; /**< d-value. */
	double k_1; /**< k_1-value. */
	double k_2; /**< k_2-value. */
	float c_p __gc[]; /**< c_p-value. */
	Vector3 *L; /**< L-vector. */
	Vector3 *V; /**< V-vector. */
	Vector3 *H; /**< H-vector. */
	Vector3 *m_imgePlanePoint; /**< point on the image plane. */

	Transferfunction *m_TransferFunc; /**< the transfer function. */

	/**
	Returns the halfway vector.
	*/
	void calculateHalfwayVector(void);
	/**
	Calculates the normal voxel to plane distance of a given point
	@param n_SampleCoords the coordinates of the point
	*/
	void calculateVoxelToPlaneDistance(Vector3* n_SampleCoords);

public:

	/**
	Standard constructor.
	*/
	PhongShadingModelSW(void);
	/**
	Constructor.
	@param n_Ambient ambient part.
	@param n_Diffuse diffuse part.
	@param n_Specular specular part.
	@param n_SpExponent specular exponent.
	*/
	PhongShadingModelSW(double n_Ambient, double n_Diffuse, double n_Specular, double n_SpExponent);
	/**
	Constructor.
	@param n_Ambient ambient part.
	@param n_Diffuse diffuse part.
	@param n_Specular specular part.
	@param n_SpExponent specular exponent.
	@param n_DepthCue1 depth cueing constant 1.
	@param n_DepthCue2 depth cueing constant 2.
	@param n_TransferFunc transfer function.
	*/
	PhongShadingModelSW(double n_Ambient, double n_Diffuse, double n_Specular, double n_SpExponent,
					    double n_DepthCue1, double n_DepthCue2/*, Vector3* n_ViewDirection, 
					    Vector3* n_ImagePlanePoint, Light* n_Light*/, Transferfunction *n_TransferFunc);

	/**
	Calculates the Color value for a given raysample.
	@param n_RaySample the specified raysample.
	@return The modified raysample.
	*/
	virtual RaySample* doShading(RaySample* n_RaySample);
	
	/**
	Sets the current view direction.
	@param n_ViewDirection the current view direction.
	*/
	__property void set_ViewingDirection(Vector3* n_ViewDirection);
	/**
	Sets an arbitrary image plane point (needed for calculation of normal voxel to plane distance).
	@param n_ImagePlanePoint an arbitrary image plane point.
	*/
	__property void set_ImagePlanePoint(Vector3* n_ImagePlanePoint);
	/**
	Sets the current light.
	@param n_Light the current light.
	*/
	__property void set_LightModel(Light* n_Light);
	/**
	Returns the transfer function.
	@return The transfer function.
	*/
	__property Transferfunction* get_TransferFunc(void);
	/**
	Returns the ambient value.
	@return The ambient value.
	*/
	__property double get_Ambient(void);
	/**
	Returns the diffuse value.
	@return The diffuse value.
	*/
	__property double get_Diffuse(void);
	/**
	Returns the specular value.
	@return The specular value.
	*/
	__property double get_Specular(void);
	/**
	Returns the specular exponent.
	@return The specular exponent.
	*/
	__property double get_SpExponent(void);
	/**
	Returns the first value for depth cueing.
	@return The first value for depth cueing.
	*/
	__property double get_DepthK1(void);
	/**
	Returns the second value for depth cueing.
	@return The second value for depth cueing.
	*/
	__property double get_DepthK2(void);
	
	/**
	Initialize viewing and lighting parameter all together to the given values.
	@param n_ViewDirection the current view direction.
	@param n_ImagePlanePoint an arbitrary image plane point.
	@param n_Light the current light.
	*/
	void setViewLightParameters(Vector3* n_ViewDirection, Vector3* n_ImagePlanePoint, Light* n_Light);
};
}
}
}
}